/*
 * Author: Utkarsh Gupta
 * Created on: June 5, 2012
 * File Name : expand_PG.cpp
 * Project Name : Automation of Folding (DDP)
 * Description : This file expands the Bipartite PG Graph.
 *        The number of nodes increase from #nodes to
 *        (#nodes + #dummy nodes). This function id used if
 *        the number of nodes are prime. Dummy nodes are
 *        added to make these composite.
 */

#include <iostream>
#include <vector>
#include "C_Functions/data_structs.h"

using std::vector;
using namespace std;

void expand_PG(parameters &PG_parameters)
{
    // Sorting the
    int pre_order = PG_parameters.order;
    int *old_conn = new int[pre_order];
    for (int i = 0; i < pre_order; i++){
        old_conn[i] = PG_parameters.hyp_conn[i];
    }

    // bi_adj_mat[ row id (for hyp nodes) ][ col id (for point nodes) ]
    vector<vector<int> > bi_adj_mat;

    bi_adj_mat.resize(PG_parameters.nodes + PG_parameters.num_dummy);
    for (int i = 0; i < PG_parameters.nodes + PG_parameters.num_dummy; ++i){
        bi_adj_mat[i].resize(PG_parameters.nodes + PG_parameters.num_dummy);
    }

    for(int col_index = 0; col_index < PG_parameters.nodes + PG_parameters.num_dummy; col_index++){
        for(int row_index = 0; row_index < PG_parameters.nodes + PG_parameters.num_dummy; row_index++){
          bi_adj_mat[row_index][col_index] = 0; // Initializing all elements to ZERO
        }
    }

    // Writing connections in the first row i.e. corresponding to the connections of HYP #0
    for (int edge_no = 0; edge_no < PG_parameters.order; edge_no++){
        bi_adj_mat[0][PG_parameters.hyp_conn[edge_no]] = 1;
    }

    // Circulating this in the UNEXPANDED Portion of the Bi Adjacency Matrix
    for (int row_index = 1; row_index < PG_parameters.nodes; row_index++ ){
        for (int col_index = 0; col_index < PG_parameters.nodes; col_index++ ){
                bi_adj_mat[row_index][col_index] = bi_adj_mat[row_index - 1][(col_index + PG_parameters.nodes -1)%PG_parameters.nodes];
        }
    }

    //Till here the Bi Adjacency Matrix is Semi-Complete

    // Circulating each '1' in the *FIRST ROW* all throughout the matrix
    for(int col_index = 0; col_index < PG_parameters.nodes + PG_parameters.num_dummy; col_index++){
        if (bi_adj_mat[0][col_index] == 1){ // some entry in First row of matrix is 1
            for(int row_index = 0; row_index < PG_parameters.nodes + PG_parameters.num_dummy; row_index++){
                bi_adj_mat[row_index][(row_index + col_index)%(PG_parameters.nodes + PG_parameters.num_dummy)] = 1; // circulating this minor diagonal
            }
        }
    }

    // Circulating each '1' in the *FIRST COLOUMN* all throughout the matrix
    for(int row_index = 0; row_index < PG_parameters.nodes + PG_parameters.num_dummy; row_index++){
        if (bi_adj_mat[row_index][0] == 1){ // some entry in *FIRST COLOUMN* of matrix is 1
            for(int col_index = 0; col_index < PG_parameters.nodes + PG_parameters.num_dummy; col_index++){
                bi_adj_mat[(row_index + col_index)%(PG_parameters.nodes + PG_parameters.num_dummy)][col_index] = 1; // circulating this minor diagonal
            }
        }
    }
    // Now we have the Completed Bi Adjency Matrix

    PG_parameters.nodes = PG_parameters.nodes + PG_parameters.num_dummy;
    PG_parameters.order = 0;

    for(int col_index = 0; col_index < PG_parameters.nodes; col_index++){
        if(bi_adj_mat[0][col_index] ==1)
            PG_parameters.order++ ;
    }

    int h_conn_num = 0;
    int p_conn_num = 0;

    PG_parameters.hyp_conn = new int[PG_parameters.order];
    PG_parameters.point_conn = new int[PG_parameters.order];

    for(int index = 0; index < PG_parameters.nodes; index++){
        if(bi_adj_mat[0][index] == 1){
            PG_parameters.hyp_conn[h_conn_num] = index;
            h_conn_num++;
        }

        if(bi_adj_mat[index][0] == 1){
            PG_parameters.point_conn[p_conn_num] = index;
            p_conn_num++;
        }
    }

    return ;
}

