/*
 * Author: Utkarsh Gupta
 * Created on: June 5, 2012
 * File Name : refinement1.cpp
 * Project Name : Automation of Folding (DDP)
 * Description : This function allows the user to
 *            dwell into Refinement 1 for the Bipartite PG.
 *            Here, User can view the connections of H/P Side
*
 */

#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include <malloc.h>
#include "C_Functions/data_structs.h"
#include "C_Functions/auxiliary_functions.h"
#include "C_Functions/Refinement_1/tool_initialize.h"
#include "C_Functions/Refinement_1/IsPrime.h"
#include "C_Functions/Refinement_1/expand_PG.h"

using std::vector;
using namespace std;

void refinement1( parameters &r){
    // for(int i = 0; i < 100; i++)
        // cout << i << " is Prime = " << IsPrime(i) << endl;

    tool_initialize (r);

    /* #The Number of nodes are prime so Expand the Graph
     * User-Input for Number of Dummy Nodes
     */
    bool usr_entry;
    r.num_dummy = 0;
    int dummy_nodes;
    usr_entry = true;
    if(IsPrime(r.nodes))
        cout << "\nNODES are Prime !! \nExpanding with Dummy Nodes is necessary.\n";

    while(usr_entry){

        if(usr_entry == true && r.num_dummy != 0 ){
            cout << "\nERROR: With " << r.num_dummy << " Dummy Nodes, ";
            cout << "#NODES = "<< r.nodes + r.num_dummy <<" would still be prime ! \n\n";
        }

        cout << "Enter the number of Dummy Nodes : " ;
        cin >> dummy_nodes;
        r.num_dummy = dummy_nodes;
        //cout << endl;
        //cout << r.nodes + r.num_dummy << endl;
        usr_entry = IsPrime(r.nodes + r.num_dummy);
        //cout << usr_entry << endl;
    }

    /* Expand the PG Graph with the dummy nodes entered above
     * Non Prime Node Case: Dummy Nodes will be 0 => No Expansion
     */
    if ( r.num_dummy != 0)
        expand_PG (r);

    string str = ( r.num_dummy == 0 ) ? "":"EXPANDED";
    cout << " * " << str << " PARAMETERS OF THE PG GRAPH\n";
    cout << " *     NODES (Total)\t\t=> " << r.nodes  << endl;
    cout << " *     DUMMY NODES\t\t=> " << r.num_dummy << endl;
    cout << " *     ORDER\t\t\t=> " << r.order << endl;
    cout << " *     HYP#0 CONNECTIONS\t=> { ";
    for(int conn_num = 0; conn_num < r.order - 1; conn_num++)
            cout << r.hyp_conn[conn_num] <<", ";
    cout << r.hyp_conn[r.order - 1] <<" }\n";
    cout << " *     POINT#0 CONNECTIONS\t=> { ";
    for(int conn_num = 0; conn_num < r.order - 1; conn_num++)
            cout << r.point_conn[conn_num] <<", ";
    cout << r.point_conn[r.order - 1] <<" }\n\n";

    /* For Folding the PG Graph - User Input for Fold Factor
     */
    r.fold_factor = 1;
    usr_entry = true;
    do{
        if(!usr_entry){
            cout << "ERROR: " << r.nodes << " NODES ";
            cout <<" can not be folded by factor " << r.fold_factor << " !\n\n";
        }

        cout << "Enter the folding factor to fold " << r.nodes << " nodes : " ;
        cin >> r.fold_factor;
        cout << endl;

        usr_entry = ( r.nodes % r.fold_factor  == 0  );
    }
    while(!usr_entry);

    /* Printing the connections of the Graph is need be
     */
    char sel;
    sel = '\0';
    while (sel != 'y' && sel != 'n' )
    {
        cout << "Press 'y' for viewing connections of PG Graph\n\t\t\tOR 'n' to SKIP to Refinement 2 : ";
        cin >> sel;
    }

    if(sel == 'y'){
        cout << "\n\n~PARAMETERS FOR THE BIPARTITE GRAPH~\n";
        cout << "\n\t - NODES = " << r.nodes <<" \n";
        cout << "\t - ORDER = " << r.order <<" \n";
        cout << "\t - FOLD FACTOR = " << r.fold_factor <<" \n";
        cout << "\t - DUMMY NODES = " << r.num_dummy <<" \n";

        cout << "\nHYPERPLANE NODE CONNECTIONS\n";
        cout << " xx\t";
        for(int edge_num = 0; edge_num < r.order; edge_num++)
            cout << "E#" << edge_num << "\t";
        cout << "\n";

        for(int node_num = 0; node_num < r.nodes; node_num++){
            cout << "H#" << node_num << "\t";
            for(int edge_num = 0; edge_num < r.order; edge_num++)
                cout << (r.hyp_conn[edge_num] + node_num)%r.nodes << "\t";
            cout << "\n";
        }

        cout << "\n\nPOINT NODE CONNECTIONS\n";
        cout << " xx\t";
        for(int edge_num = 0; edge_num < r.order; edge_num++)
            cout << "E#" << edge_num << "\t";
        cout << "\n";

        for(int node_num = 0; node_num < r.nodes; node_num++){
            cout << "P#" << node_num << "\t";
            for(int edge_num = 0; edge_num < r.order; edge_num++)
                cout << (r.point_conn[edge_num] + node_num)%r.nodes << "\t";
            cout << "\n";
        }
    }

    r.folded_nodes = r.nodes/r.fold_factor;

    r.h0_to_p = new int[r.folded_nodes]; // H->P connections in a FOLDED PG GRAPH
                               // Array Index signifies the FOLDED POINT NODE INDEX the H0 is connected to
                               // The value in the array element signifies the *number of connections* (0/1/2)
                               // between Hyperplane Node 0 and Folded POINT NODE Index (array index)

    r.p0_to_h = new int[r.folded_nodes]; // P->H connections in a FOLDED PG GRAPH
                               // Array Index signifies the FOLDED HYPERPLANE NODE INDEX the P0 is connected to
                               // The value in the array element signifies the *number of connections* (0/1/2)
                               // between Point Node 0 and Folded HYPERPLANE NODE Index (array index)

    // Initializing all connections to *ZERO*
    for(int node_num = 0; node_num < r.folded_nodes; node_num++){
        r.h0_to_p[node_num] = 0;
        r.p0_to_h[node_num] = 0;
    }

    /*The part of this function is used for generating the following:
            <Looking from the HYPERPLANE SIDE>
            - r.h0_to_p [ X ] = '0' if No connection between Hyp #0 and Point #X
                              '1' if Single connection between Hyp #0 and Point #X
                              '2' if Two connections between Hyp #0 and Point #X
                                  Only in the case when 2 edges in same cycle map
                                  to the same Folded node

            <Looking from the POINT SIDE>
            - r.p0_to_h [ Y ]    = '0' if No connection between Point #0 and Hyp #Y
                              '1' if Single connection between Point #0 and Hyp #Y
                              '2' if Two connections between Point #0 and Hyp #Y
                                  Only in the case when 2 edges in same cycle map
                                  to the same Folded node                            */

    for (int i =0; i < r.order; i = i+2){
        if(r.h0_to_p[r.hyp_conn[i] % r.folded_nodes] < 1){
            r.h0_to_p[r.hyp_conn[i] % r.folded_nodes] = 1;
        }

        if(i+1 < r.order && r.h0_to_p[r.hyp_conn[i+1] % r.folded_nodes] < 1){
            r.h0_to_p[r.hyp_conn[i+1] % r.folded_nodes] = 1;
        }
        else if(i+1 < r.order && (r.hyp_conn[i+1] % r.folded_nodes) == (r.hyp_conn[i] % r.folded_nodes)){
            r.h0_to_p[r.hyp_conn[i+1] % r.folded_nodes] = 2;
        }

        if(r.p0_to_h[r.point_conn[i] % r.folded_nodes] < 1){
            r.p0_to_h[r.point_conn[i] % r.folded_nodes] = 1;
        }

        if(i+1 < r.order && r.p0_to_h[r.point_conn[i+1] % r.folded_nodes] < 1){
            r.p0_to_h[r.point_conn[i+1] % r.folded_nodes] = 1;
        }
        else if(i+1 < r.order && (r.point_conn[i+1] % r.folded_nodes) == (r.point_conn[i] % r.folded_nodes)){
            r.p0_to_h[r.point_conn[i+1] % r.folded_nodes] = 2;
        }
    }

    // Number of edges per node on each side of the FOLDED PG GRAPH
    r.num_h_edges = 0;    // <Looking from the HYPERPLANE SIDE>
    r.num_p_edges = 0;    // <Looking from the POINT SIDE>

    // Total edges *from Hyperplane Nodes* = SUM(r.h0_to_p array elements)
    for (int node_num = 0; node_num < r.folded_nodes; node_num++){
        // Total edges *from Hyp->Point* = SUM(r.h0_to_p array elements)
        r.num_h_edges = r.num_h_edges + r.h0_to_p[node_num];

        // Total edges *from Point->Hyp* = SUM(r.p0_to_h array elements)
        r.num_p_edges = r.num_p_edges + r.p0_to_h[node_num];
    }

    return;
}

