/*
 * Author: Utkarsh Gupta
 * Created on: June 5, 2012
 * File Name : fold_PG.cpp
 * Project Name : Automation of Folding (DDP)
 * Description : This file creates the Topmost Module File
 *            which is "fold_PG"
 */

#include <iostream>
#include <fstream>
#include <cmath>
#include <string.h>
#include <vector>
#include "C_Functions/data_structs.h"
#include "C_Functions/auxiliary_functions.h"

using std::vector;
using namespace std;

void make_fold_pg(parameters PG_parameters, int addr_bits, const char * outputfile)
{
    ofstream outfile("Generated_Files/Portions/Fold_PG/switch2.vhd");
    ofstream outfile2("Generated_Files/Portions/Fold_PG/switch4.vhd");
    ofstream outfile3("Generated_Files/Portions/Fold_PG/switch6.vhd");
    ofstream outfile4("Generated_Files/Portions/Fold_PG/switch8.vhd");

    for(int edge_num = 0; edge_num < PG_parameters.num_p_edges-1; edge_num++){
        outfile << "           out_val_" << edge_num + 1 << "     : OUT STD_LOGIC;\n";
        outfile4 << "           in_val_" << edge_num + 1 << "         : IN STD_LOGIC;\n";
    }
    outfile << "           out_val_" << PG_parameters.num_p_edges << "     : OUT STD_LOGIC\n         );\n";
    outfile4 << "           in_val_" << PG_parameters.num_p_edges << "         : IN STD_LOGIC;\n";

    for(int edge_num = 0; edge_num < PG_parameters.num_h_edges-1; edge_num++){
        outfile2 << "           out_val_" << edge_num + 1 << "     : OUT STD_LOGIC;\n";
        outfile3 << "           in_val_" << edge_num + 1<< "         : IN STD_LOGIC;\n";
    }
    outfile2 << "           out_val_" << PG_parameters.num_h_edges << "     : OUT STD_LOGIC\n         );\n";
    outfile3 << "           in_val_" << PG_parameters.num_h_edges << "         : IN STD_LOGIC;\n";

    outfile4.close();
    outfile3.close();
    outfile2.close();
    outfile.close();

    outfile.open(outputfile);
    outfile.close();

    append ("HDL_partitioned_files/Fold_PG/fold_pg1.vhd", outputfile);
    append ("HDL_partitioned_files/Fold_PG/switch1.vhd", outputfile);
    append ("Generated_Files/Portions/Fold_PG/switch2.vhd", outputfile);
    append ("HDL_partitioned_files/Fold_PG/switch3.vhd", outputfile);
    append ("Generated_Files/Portions/Fold_PG/switch4.vhd", outputfile);
    append ("HDL_partitioned_files/Fold_PG/switch5.vhd", outputfile);
    append ("Generated_Files/Portions/Fold_PG/switch6.vhd", outputfile);
    append ("HDL_partitioned_files/Fold_PG/switch7.vhd", outputfile);
    append ("Generated_Files/Portions/Fold_PG/switch8.vhd", outputfile);
    append ("HDL_partitioned_files/Fold_PG/switch9.vhd", outputfile);

    append ("HDL_partitioned_files/Fold_PG/fold_pg2.vhd", outputfile);

    outfile.open("Generated_Files/Portions/Fold_PG/start_gen.vhd");
    outfile << "--------------------------------------------------------------------------\n";
    outfile << "BEGIN\n-- Generate all the functional and memory units\nG1:FOR i IN " << PG_parameters.folded_nodes - 1 << " DOWNTO 0 GENERATE\n\n";
    outfile.close();

    outfile.open("Generated_Files/Portions/Fold_PG/cslv.vhd");
    outfile << "                                      CONV_STD_LOGIC_VECTOR( i, " << bin_digits(PG_parameters.folded_nodes - 1) << " ),\n";
    outfile.close();

    append( "Generated_Files/Portions/Fold_PG/start_gen.vhd", outputfile );
    append( "HDL_partitioned_files/Fold_PG/hu_portmap.vhd", outputfile );
    append( "HDL_partitioned_files/Fold_PG/pu_portmap.vhd", outputfile );

    append( "HDL_partitioned_files/Fold_PG/hu_mem1.vhd", outputfile );
    append( "Generated_Files/Portions/Fold_PG/cslv.vhd", outputfile );
    append( "HDL_partitioned_files/Fold_PG/hu_mem2.vhd", outputfile );

    append( "HDL_partitioned_files/Fold_PG/pu_mem1.vhd", outputfile );
    append( "Generated_Files/Portions/Fold_PG/cslv.vhd", outputfile );
    append( "HDL_partitioned_files/Fold_PG/pu_mem2.vhd", outputfile );

    outfile.open("Generated_Files/Portions/Fold_PG/hps_conn.vhd");
    outfile2.open("Generated_Files/Portions/Fold_PG/hms_conn.vhd");
    outfile3.open("Generated_Files/Portions/Fold_PG/pps_conn.vhd");
    outfile4.open("Generated_Files/Portions/Fold_PG/pms_conn.vhd");

    for(int node_num = 0; node_num < PG_parameters.folded_nodes; node_num++)
    {
        outfile << "h_p_switch_" << node_num << ":  h_p_switch PORT MAP( clk, en_p_switch_b, pat_id_b,\n";
        outfile << "                                \"" << binary(node_num, PG_parameters.folded_nodes - 1) << "\", -- " << node_num << "th unit\n";
        for ( int conn = 0; conn < PG_parameters.num_h_edges; conn++)
        {
            outfile << "                                c_to_b_wires( " << PG_parameters.H_PPU[node_num].ip_ports[conn] << " ),\n";
        }
        outfile << "                                bfu_in1( " << node_num << " ), bfu_in2( " << node_num << " )\n";
        outfile << "                                  );\n\n";

        outfile2 << "h_m_switch_" << node_num << ":  h_m_switch   PORT MAP( clk, en_m_switch_b, pat_id_b,\n";
        outfile2 << "                                \"" << binary(node_num, PG_parameters.folded_nodes - 1) << "\", -- " << node_num << "th unit\n";
        outfile2 << "                                bmu_out1(" << node_num << "), bmu_out2(" << node_num << "),\n";
        for ( int conn = 0; conn < PG_parameters.num_p_edges - 1; conn++)
        {
            outfile2 << "                                b_to_c_wires( " << PG_parameters.H_PMU[node_num].op_ports[conn] << " ),\n";
                                                                                // when you get out of here see what you have done
        }
        outfile2 << "                                b_to_c_wires( " << PG_parameters.H_PMU[node_num].op_ports[ PG_parameters.num_p_edges - 1] << " )\n";
        outfile2 << "                              );\n";

        outfile3 << "p_p_switch_" << node_num << ":  p_p_switch PORT MAP( clk, en_p_switch_c, pat_id_c,\n";
        outfile3 << "                                \"" << binary(node_num, PG_parameters.folded_nodes - 1) << "\", -- " << node_num << "th unit\n";
        for ( int conn = 0; conn < PG_parameters.num_p_edges; conn++)
        {
            outfile3 << "                                b_to_c_wires( " << PG_parameters.P_PPU[node_num].ip_ports[conn] << " ),\n";
        }
        outfile3 << "                                cfu_in1( " << node_num << " ), cfu_in2( " << node_num << " )\n";
        outfile3 << "                                  );\n\n";

        outfile4 << "p_m_switch_" << node_num << ":  p_m_switch   PORT MAP( clk, en_m_switch_c, pat_id_c,\n";
        outfile4 << "                                \"" << binary(node_num, PG_parameters.folded_nodes - 1) << "\", -- " << node_num << "th unit\n";
        outfile4 << "                                cmu_out1(" << node_num << "), cmu_out2(" << node_num << "),\n";
        for ( int conn = 0; conn < PG_parameters.num_h_edges - 1; conn++)
        {
            outfile4 << "                                c_to_b_wires( " << PG_parameters.P_PMU[node_num].op_ports[conn] << " ),\n";
                                                                                // when you get out of here see what you have done
        }
           outfile4 << "                                c_to_b_wires( " << PG_parameters.P_PMU[node_num].op_ports[ PG_parameters.num_h_edges - 1] << " )\n";
        outfile4 << "                              );\n";
    }

    outfile.close();
    outfile2.close();
    outfile3.close();
    outfile4.close();

    append("Generated_Files/Portions/Fold_PG/hps_conn.vhd", outputfile );
    append("Generated_Files/Portions/Fold_PG/hms_conn.vhd", outputfile );
    append("Generated_Files/Portions/Fold_PG/pps_conn.vhd", outputfile );
    append("Generated_Files/Portions/Fold_PG/pms_conn.vhd", outputfile );

    outfile.open("Generated_Files/Portions/Fold_PG/fold_pg3b.vhd");
    int bit_index = 28 + bin_digits(((PG_parameters.order +  (PG_parameters.order%2))/2) - 1) - 1;
    if ( bit_index != 28)
        outfile << "pat_id_b         <= microcode( " << bit_index << " downto 28 );\n";
    else
        outfile << "pat_id_b         <= microcode( 28 );\n";
    bit_index = bit_index + 2;
    int temp = bit_index;

    bit_index = temp + bin_digits( PG_parameters.fold_factor - 1) - 1;
    if ( bit_index != temp )
        outfile << "fold_indx_b      <= microcode( " << bit_index << " downto " << temp << " );\n";
    else
        outfile << "fold_indx_b      <= microcode( " << temp << " );\n";
    bit_index = bit_index + 2;
    temp = bit_index;

    bit_index = temp + bin_digits(((PG_parameters.order +  (PG_parameters.order%2))/2) - 1) - 1;
    if ( bit_index != temp )
        outfile << "pat_id_c         <= microcode( " << bit_index << " downto " << temp << " );\n";
    else
        outfile << "pat_id_c         <= microcode( " << temp << " );\n";
    bit_index = bit_index + 2;
    temp = bit_index;

    bit_index = temp + bin_digits( PG_parameters.fold_factor - 1 ) - 1;
    if ( bit_index != temp )
        outfile << "fold_indx_c      <= microcode( " << bit_index << " downto " << temp << " );\n\n";
    else
        outfile << "fold_indx_c      <= microcode( " << temp << " );\n\n";
    outfile.close();

    append("HDL_partitioned_files/Fold_PG/fold_pg3.vhd", outputfile );
    append("Generated_Files/Portions/Fold_PG/fold_pg3b.vhd", outputfile );

    outfile.open("Generated_Files/Portions/Fold_PG/code_check.vhd");
    outfile << "-- Storing an estimated codeword, in case during next 'check' phase, all parity checks turn out to be satisfied.\n\n";

    for (int x = 0; x<PG_parameters.fold_factor; x++)
    {
      outfile << "be" << x+1 << ": rxvd_code( "<<((x+1)*PG_parameters.folded_nodes)-1<<" downto "<<x*PG_parameters.folded_nodes<<" ) <= rxvd_fold WHEN ( en_out_b = '1' AND fold_indx_b = \""<<binary(x,PG_parameters.fold_factor)<<"\" ) ELSE rxvd_code( "<<((x+1)*PG_parameters.folded_nodes)-1<<" downto "<<x*PG_parameters.folded_nodes<<" );\n\n";
    }

    outfile << "-- Accumulating all parity check outputs, to see if the decoding has converged.\n\n";

    for (int x = 0; x<PG_parameters.fold_factor; x++)
    {
      outfile << "pc" << x+1 << ": pcheck_out( "<<((x+1)*PG_parameters.folded_nodes)-1<<" downto "<<x*PG_parameters.folded_nodes<<" ) <= cmu_in1 WHEN ( clk'EVENT AND clk = '1' AND en_out_c = '1' AND fold_indx_c = \""<<binary(x,PG_parameters.fold_factor)<<"\" ) ELSE pcheck_out( "<<((x+1)*PG_parameters.folded_nodes)-1<<" downto "<<x*PG_parameters.folded_nodes<<" );\n\n";
    }

    outfile.close();

    append("Generated_Files/Portions/Fold_PG/code_check.vhd", outputfile );

    outfile.open("Generated_Files/Portions/Fold_PG/process1.vhd");

    outfile << "------------process for multiplexing intrinsic information ----------------\n";
    outfile << "-- Providing the intrinsic information half a clock cycle ahead, to avoid setup time violation.\n";
    outfile << "PROCESS( clk )\nBEGIN\n    IF( clk'EVENT AND clk = '0' ) THEN\n";
    outfile << "      IF( en_first_time_b = '1' ) THEN\n        ";
    for(int i = 0; i<PG_parameters.fold_factor; i++)
    {
      outfile <<"IF( fold_indx_b = \""<<binary(i,PG_parameters.fold_factor)<<"\" ) THEN\n";
      for(int j = 0; j<PG_parameters.folded_nodes; j++)
      {
        outfile << "         fold_intr_in( "<<j<<" ) <= intr_in( "<<i*PG_parameters.folded_nodes+j<<" );\n";
      }
      outfile <<"        ELS";
    }

    outfile <<"E\n";
    for(int i = 0; i<PG_parameters.folded_nodes; i++)
    {
      outfile << "         fold_intr_in( "<< i <<" ) <= \"ZZZZZZZZZ\";\n";
    }

    outfile<<"        END IF;\n";
    outfile<<"      END IF;\n";
    outfile<<"    END IF;\n";

    outfile <<"END PROCESS;\n";
    outfile.close();

    string q = "";
    string x = "";
    for(int i = 0; i<PG_parameters.folded_nodes * PG_parameters.fold_factor; i++)
    {
      q = q + "0";
    }

    for(int i = 0; i<addr_bits; i++)
    {
      x = x + "0";
    }

    outfile.open("Generated_Files/Portions/Fold_PG/process2b.vhd");
    outfile2.open("Generated_Files/Portions/Fold_PG/process2d.vhd");

    outfile << " = \"" << q;
    outfile2 << " = \"" << x;

    outfile.close();
    outfile2.close();

    append("Generated_Files/Portions/Fold_PG/process1.vhd", outputfile);
    append("HDL_partitioned_files/Fold_PG/process2a.vhd", outputfile );
    append("Generated_Files/Portions/Fold_PG/process2b.vhd", outputfile );
    append("HDL_partitioned_files/Fold_PG/process2c.vhd", outputfile );
    append("Generated_Files/Portions/Fold_PG/process2d.vhd", outputfile );
    append("HDL_partitioned_files/Fold_PG/process2e.vhd", outputfile );

}


