#include <iostream>
#include <fstream>
#include <cmath>
#include <string.h>
#include <vector>
#include "C_Functions/auxiliary_functions.h"

using namespace std;

void make_microcode(int **microcode, int folds, int patterns);

void make_ctrl_str( int order, int fold_factor){
    int patterns = ((order + (order%2))/2);
    int pattern_bits = bin_digits( patterns - 1);
    int fold_bits = bin_digits( fold_factor - 1);
    int microcode_bits = 31 + 2*fold_bits + 2*pattern_bits ;
    int total_cycles = fold_factor + 4 + fold_factor*patterns + fold_factor + fold_factor*patterns + 4 + fold_factor*patterns + fold_factor*patterns + 2;

    int **microcode = new int *[microcode_bits];
    for ( int i = 0; i < microcode_bits; i++){
        microcode[i] = new int [total_cycles];
        for ( int cycle_num = 0; cycle_num < total_cycles; cycle_num++)
            microcode[i][cycle_num] = 0;
    }

    /*
        STILL TO MAKE THE MICROCODE
    */
    make_microcode(microcode, fold_factor, patterns);

    ofstream outfile("Generated_Files/Portions/Control_store/ctrl_str_mid.vhd");
    for ( int cycle_num = 0; cycle_num < total_cycles; cycle_num++){
        outfile << "        WHEN \"" << binary( cycle_num, total_cycles - 1 )  << "\"   => dout <= \"";

        for ( int i = microcode_bits - 1; i >= 0; i--)
            outfile << microcode[i][cycle_num] ;

        outfile << "\";\n";
    }
    outfile << "        WHEN  OTHERS    => dout <= \"";
    for ( int i = 0; i < microcode_bits; i++)
            outfile << 0 ;
    outfile << "\";\n";
    outfile.close();

    outfile.open("HDL_Final_Modules/control_store.vhd");
    outfile.close();

    append( "HDL_partitioned_files/Control_store/ctrl_str_start.vhd", "HDL_Final_Modules/control_store.vhd");
    append( "Generated_Files/Portions/Control_store/ctrl_str_mid.vhd", "HDL_Final_Modules/control_store.vhd");
    append( "HDL_partitioned_files/Control_store/ctrl_str_end.vhd", "HDL_Final_Modules/control_store.vhd");

    return;
}

void make_microcode(int **microcode, int folds, int patterns){
    int hyp_fold_index = 28 + bin_digits(patterns - 1) - 1 + 2;
    int point_pat_index = hyp_fold_index + bin_digits(folds - 1) - 1 + 2;
    int point_fold_index = point_pat_index + bin_digits(patterns - 1) - 1 + 2;

    fill_array( microcode[14], 0, folds, 1 ); // en_first_time

    fill_seq2( microcode, 0, folds, folds, hyp_fold_index , bin_digits(folds - 1)); // fold_index_Hyp

    microcode[3][0] = 1; // rst_Point

    fill_array( microcode[4], folds, folds*patterns, 1 );  // en_mem_Point
    fill_array( microcode[5], folds, folds*patterns, 1 ); // en_mem_rwbar1_Point
    fill_array( microcode[6], folds, folds*patterns, 1 );  // en_mem_rwbar2_Point

    fill_array( microcode[7], folds + 2, folds*patterns, 1 );  // en_m_switch_Point

    fill_seq3( microcode, folds + 2, folds*patterns, folds, point_pat_index, bin_digits(patterns - 1) );  // pattern_index_Point

    fill_array( microcode[13], folds + 3, folds*patterns, 1 );  // en_p_switch_Hyp
    fill_seq3( microcode, folds + 3, folds*patterns, folds , 28, bin_digits(patterns - 1));  // patterns_index_Hyp

    fill_array( microcode[17], folds + 4, folds*patterns, 1 );  // en_in_Hyp

    fill_seq2( microcode, folds + 4, folds*patterns, folds, hyp_fold_index, bin_digits(folds - 1) );  // fold_index_Hyp

    int cycles_completed;

    microcode[15][folds + 3] = 1;  // rst_Hyp

    fill_array( microcode[18], folds + folds*patterns + 4, folds, 1 );  // EXTRA CONTROL SIGNAL: en_flip_Hyp
    fill_seq2( microcode, folds + folds*patterns + 4, folds, folds, hyp_fold_index, bin_digits(folds - 1) );  // fold_index_Hyp

    fill_array( microcode[19], folds + folds*patterns + folds + 4, folds*patterns, 1 );  // en_out_Hyp
    fill_array( microcode[21], folds + folds*patterns + folds + 4, folds*patterns, 1 );  // en_mem_Hyp
    fill_seq2( microcode, folds + folds*patterns + folds + 4, folds*patterns, folds, hyp_fold_index, bin_digits(folds - 1) );  // fold_index_Hyp

    cycles_completed = folds + folds*patterns + folds + folds*patterns + 4;

    cycles_completed++;

    microcode[3][cycles_completed] = 1; // rst_Point
    fill_array( microcode[21], cycles_completed, folds*patterns, 1 );  // en_mem_Hyp
    fill_array( microcode[22], cycles_completed, folds*patterns, 1 );  // en_mem_rwbar1_Hyp
    fill_array( microcode[23], cycles_completed, folds*patterns, 1 );  // en_mem_rwbar1_Hyp

    fill_array( microcode[24], cycles_completed + 2, folds*patterns, 1 );  // en_m_switch_Hyp
    fill_seq3( microcode, cycles_completed + 2, folds*patterns, folds , 28, bin_digits(patterns - 1) );  // pattern_index_Hyp

    fill_array( microcode[0], cycles_completed + 3, folds*patterns, 1 );  // en_p_switch_Point
    fill_seq3( microcode, cycles_completed + 3, folds*patterns, folds, point_pat_index, bin_digits(patterns - 1) );  // pattern_index_Point

    fill_array( microcode[1], cycles_completed + 4, folds*patterns, 1 );  // en_in_Point
    fill_seq2( microcode, cycles_completed + 4, folds*patterns, folds, point_fold_index, bin_digits(folds - 1) );  // fold_index_Point

    microcode[26][cycles_completed + folds*patterns + folds + 4] = 1; // decide_cword
    fill_array( microcode[2], cycles_completed + folds*patterns + 4, folds*patterns, 1 );  // en_out_Point
    fill_array( microcode[4], cycles_completed + folds*patterns + 4, folds*patterns, 1 );  // en_mem_Point
    fill_seq2( microcode, cycles_completed + folds*patterns + 4, folds*patterns, folds, point_fold_index, bin_digits(folds - 1) );  // fold_index_Point

    return;
}
