/*
 * Author: Utkarsh Gupta
 * Created on: June 5, 2012
 * File Name : make_node_module.cpp
 * Project Name : Automation of Folding (DDP)
 * Description : This file creates the Node Module VHDL Files
 *            i.e. the Hyperplane Node Module File and the Point
 *            Node Module File
 */

#include <iostream>
#include <fstream>
#include <cmath>
#include <string.h>
#include <vector>
#include "C_Functions/auxiliary_functions.h"

using std::vector;
using namespace std;

void make_hyp_module(int fold_factor, const char *outputfile)
{
    ofstream outfile("Generated_Files/Portions/Node_Modules/hm2.vhd");
    ofstream outfile_1("Generated_Files/Portions/Node_Modules/hm2a.vhd");
    ofstream outfile_2("Generated_Files/Portions/Node_Modules/hm2b.vhd");
    ofstream outfile2("Generated_Files/Portions/Node_Modules/hm4.vhd");
    ofstream outfile_3("Generated_Files/Portions/Node_Modules/hm5b.vhd");
    ofstream outfile3("Generated_Files/Portions/Node_Modules/hm6.vhd");
    ofstream outfile4("Generated_Files/Portions/Node_Modules/hm8.vhd");
    ofstream outfile5("Generated_Files/Portions/Node_Modules/hm10.vhd");

    outfile << "\tSIGNAL ";
    outfile_1 << "\tSIGNAL ";
    outfile_2 << "\tSIGNAL ";
    outfile2 << "        ";
    outfile3 << "        ";
    outfile4 << "        ";
    outfile5 << "        ";
    for(int fold_num = 0; fold_num < fold_factor - 1; fold_num++){
        outfile << "bit_estimate_" << fold_num << ", ";
        outfile_1 << "no_zeroes_" << fold_num << ", ";
        outfile_2 << "no_ones_" << fold_num << ", ";

        outfile2 << "IF( fold_indx = \"" << binary(fold_num, fold_factor - 1) << "\" ) THEN\n";
        outfile2 <<"          IF( ( in_bit1, in_bit2 ) = \"11\" ) THEN\n";
        outfile2 <<"            no_ones_" << fold_num << "     <= no_ones_" << fold_num << " + 2;\n";
        outfile2 <<"          ELSIF( ( in_bit1, in_bit2 ) = \"10\" ) THEN\n";
        outfile2 <<"            no_zeroes_" << fold_num << "   <= no_zeroes_" << fold_num << " + 1;\n";
        outfile2 <<"            no_ones_" << fold_num << "     <= no_ones_" << fold_num << " + 1;\n";
        outfile2 <<"          ELSIF( ( in_bit1, in_bit2 ) = \"01\" ) THEN\n";
        outfile2 <<"            no_zeroes_" << fold_num << "   <= no_zeroes_" << fold_num << " + 1;\n";
        outfile2 <<"            no_ones_" << fold_num << "     <= no_ones_" << fold_num << " + 1;\n";
        outfile2 <<"          ELSIF( ( in_bit1, in_bit2 ) = \"00\" ) THEN\n";
        outfile2 <<"            no_zeroes_" << fold_num << "   <= no_zeroes_" << fold_num << " + 2;\n";
        outfile2 <<"          ELSE\n";
        outfile2 <<"            IF( in_bit1 = '1' ) THEN -- in_bit2 may be Z, X etc.\n";
        outfile2 <<"              no_ones_" << fold_num << "   <= no_ones_" << fold_num << " + 1;\n";
        outfile2 <<"            ELSIF( in_bit1 = '0' ) THEN -- in_bit2 may be Z, X etc.\n";
        outfile2 <<"              no_zeroes_" << fold_num << " <= no_zeroes_" << fold_num << " + 1;\n";
        outfile2 <<"            END IF;\n\n";
        outfile2 <<"            IF( in_bit2 = '1' ) THEN -- in_bit1 may be Z, X etc.\n";
        outfile2 <<"              no_ones_" << fold_num << " <= no_ones_" << fold_num << " + 1;\n";
        outfile2 <<"            ELSIF( in_bit2 = '0' ) THEN -- in_bit1 may be Z, X etc.\n";
        outfile2 <<"              no_zeroes_" << fold_num << " <= no_zeroes_" << fold_num << " + 1;\n";
        outfile2 <<"            END IF;\n";
        outfile2 <<"          END IF;\n";
        outfile2 << "        ELS";

        outfile_3 << "\t\t\tno_ones_" << fold_num << "\t<= 0;\n";
        outfile_3 << "\t\t\tno_zeroes_" << fold_num << "\t<= 0;\n";

        outfile3 << "IF( fold_indx = \"" << binary(fold_num, fold_factor - 1) << "\" ) THEN\n";
        outfile3 << "          IF( intrinsic_vec(8) = '1' ) THEN -- negative value\n";
        outfile3 << "            bit_estimate_" << fold_num << " <= '1';\n";
         outfile3 << "          ELSIF( intrinsic_vec(8) = '0' ) THEN -- positive value\n";
        outfile3 << "            bit_estimate_" << fold_num << " <= '0';\n";
        outfile3 << "          END IF;\n";
        outfile3 << "        ELS";

        outfile4 << "IF( fold_indx = \"" << binary(fold_num, fold_factor - 1) << "\" ) THEN\n";
        outfile4 << "          IF( no_ones_" << fold_num << " >= no_zeroes_" << fold_num << " ) THEN -- more parity check failures\n";
        outfile4 << "            bit_estimate_" << fold_num << " <= NOT( bit_estimate_" << fold_num << " );\n";
        outfile4 << "          ELSE\n";
        outfile4 << "            bit_estimate_" << fold_num << " <= bit_estimate_" << fold_num << ";\n";
        outfile4 << "          END IF;\n";
        outfile4 << "        ELS";

        outfile5 << "IF( fold_indx = \"" << binary(fold_num, fold_factor - 1) << "\" ) THEN\n";
        outfile5 << "          bit_estimate  <= bit_estimate_" << fold_num << ";\n";
        outfile5 << "          out_bit1      <= bit_estimate_" << fold_num << ";\n";
        outfile5 << "          out_bit2      <= bit_estimate_" << fold_num << ";\n";
        outfile5 << "        ELS";
    }

    outfile << "bit_estimate_" << fold_factor - 1 << "\t : STD_LOGIC;\n";
    outfile_1 << "no_zeroes_" << fold_factor - 1 << "\t : INTEGER;\n";
    outfile_2 << "no_ones_" << fold_factor - 1 << "\t : INTEGER;\n";

    outfile2 << "IF( fold_indx = \"" << binary(fold_factor - 1, fold_factor - 1) << "\" ) THEN\n";
    outfile2 <<"          IF( ( in_bit1, in_bit2 ) = \"11\" ) THEN\n";
    outfile2 <<"            no_ones_" << fold_factor - 1 << "     <= no_ones_" << fold_factor - 1 << " + 2;\n";
    outfile2 <<"          ELSIF( ( in_bit1, in_bit2 ) = \"10\" ) THEN\n";
    outfile2 <<"            no_zeroes_" << fold_factor - 1 << "   <= no_zeroes_" << fold_factor - 1 << " + 1;\n";
    outfile2 <<"            no_ones_" << fold_factor - 1 << "     <= no_ones_" << fold_factor - 1 << " + 1;\n";
    outfile2 <<"          ELSIF( ( in_bit1, in_bit2 ) = \"01\" ) THEN\n";
    outfile2 <<"            no_zeroes_" << fold_factor - 1 << "   <= no_zeroes_" << fold_factor - 1 << " + 1;\n";
    outfile2 <<"            no_ones_" << fold_factor - 1 << "     <= no_ones_" << fold_factor - 1 << " + 1;\n";
    outfile2 <<"          ELSIF( ( in_bit1, in_bit2 ) = \"00\" ) THEN\n";
    outfile2 <<"            no_zeroes_" << fold_factor - 1 << "   <= no_zeroes_" << fold_factor - 1 << " + 2;\n";
    outfile2 <<"          ELSE\n";
    outfile2 <<"            IF( in_bit1 = '1' ) THEN -- in_bit2 may be Z, X etc.\n";
    outfile2 <<"              no_ones_" << fold_factor - 1 << "   <= no_ones_" << fold_factor - 1 << " + 1;\n";
    outfile2 <<"            ELSIF( in_bit1 = '0' ) THEN -- in_bit2 may be Z, X etc.\n";
    outfile2 <<"              no_zeroes_" << fold_factor - 1 << " <= no_zeroes_" << fold_factor - 1 << " + 1;\n";
    outfile2 <<"            END IF;\n\n";
    outfile2 <<"            IF( in_bit2 = '1' ) THEN -- in_bit1 may be Z, X etc.\n";
    outfile2 <<"              no_ones_" << fold_factor - 1 << " <= no_ones_" << fold_factor - 1 << " + 1;\n";
    outfile2 <<"            ELSIF( in_bit2 = '0' ) THEN -- in_bit1 may be Z, X etc.\n";
    outfile2 <<"              no_zeroes_" << fold_factor - 1 << " <= no_zeroes_" << fold_factor - 1 << " + 1;\n";
    outfile2 <<"            END IF;\n";
    outfile2 <<"          END IF;\n";
    outfile2 << "        END IF;\n";

    outfile_3 << "\t\t\tno_ones_" << fold_factor - 1 << "\t<= 0;\n";
    outfile_3 << "\t\t\tno_zeroes_" << fold_factor - 1 << "\t<= 0;\n";

    outfile3 << "IF( fold_indx = \"" << binary(fold_factor - 1, fold_factor - 1) << "\" ) THEN\n";
    outfile3 << "          IF( intrinsic_vec(8) = '1' ) THEN -- negative value\n";
    outfile3 << "            bit_estimate_" << fold_factor - 1 << " <= '1';\n";
    outfile3 << "          ELSIF( intrinsic_vec(8) = '0' ) THEN -- positive value\n";
    outfile3 << "            bit_estimate_" << fold_factor - 1 << " <= '0';\n";
    outfile3 << "          END IF;\n";
    outfile3 << "        END IF;\n";

    outfile4 << "IF( fold_indx = \"" << binary(fold_factor - 1, fold_factor - 1) << "\" ) THEN\n";
    outfile4 << "          IF( no_ones_" << fold_factor - 1 << " >= no_zeroes_" << fold_factor - 1 << " ) THEN -- more parity check failures\n";
    outfile4 << "            bit_estimate_" << fold_factor - 1 << " <= NOT( bit_estimate_" << fold_factor - 1 << " );\n";
    outfile4 << "          ELSE\n";
    outfile4 << "            bit_estimate_" << fold_factor - 1 << " <= bit_estimate_" << fold_factor - 1 << ";\n";
    outfile4 << "          END IF;\n";
    outfile4 << "        END IF;\n";

    outfile5 << "IF( fold_indx = \"" << binary(fold_factor - 1, fold_factor - 1) << "\" ) THEN\n";
    outfile5 << "          bit_estimate  <= bit_estimate_" << fold_factor - 1 << ";\n";
    outfile5 << "          out_bit1      <= bit_estimate_" << fold_factor - 1 << ";\n";
    outfile5 << "          out_bit2      <= bit_estimate_" << fold_factor - 1 << ";\n";
    outfile5 << "        END IF;\n";

    outfile5.close();
    outfile4.close();
    outfile3.close();
    outfile_3.close();
    outfile2.close();
    outfile_2.close();
    outfile_1.close();
    outfile.close();

    outfile.open(outputfile);
    outfile.close();

    append ("HDL_partitioned_files/Node_Modules/hm1.vhd", outputfile);
    append ("Generated_Files/Portions/Node_Modules/hm2a.vhd", outputfile);
    append ("Generated_Files/Portions/Node_Modules/hm2b.vhd", outputfile);
    append ("Generated_Files/Portions/Node_Modules/hm2.vhd", outputfile);
    append ("HDL_partitioned_files/Node_Modules/hm3.vhd", outputfile);
    append ("Generated_Files/Portions/Node_Modules/hm4.vhd", outputfile);
    append ("HDL_partitioned_files/Node_Modules/hm5a.vhd", outputfile);
    append ("Generated_Files/Portions/Node_Modules/hm5b.vhd", outputfile);
    append ("HDL_partitioned_files/Node_Modules/hm5c.vhd", outputfile);
    append ("Generated_Files/Portions/Node_Modules/hm6.vhd", outputfile);
    append ("HDL_partitioned_files/Node_Modules/hm7.vhd", outputfile);
    append ("Generated_Files/Portions/Node_Modules/hm8.vhd", outputfile);
    append ("HDL_partitioned_files/Node_Modules/hm9.vhd", outputfile);
    append ("Generated_Files/Portions/Node_Modules/hm10.vhd", outputfile);
    append ("HDL_partitioned_files/Node_Modules/hm11.vhd", outputfile);

    return;
}

void make_point_module(int fold_factor, const char *outputfile)
{
    ofstream outfile("Generated_Files/Portions/Node_Modules/pm2.vhd");
    ofstream outfile2("Generated_Files/Portions/Node_Modules/pm4.vhd");
    ofstream outfile3("Generated_Files/Portions/Node_Modules/pm6.vhd");
    ofstream outfile4("Generated_Files/Portions/Node_Modules/pm8.vhd");

    outfile3 << "          ";
    outfile4 << "        ";
    for(int fold_num = 0; fold_num < fold_factor - 1; fold_num++){
        outfile << "check_res" << fold_num << ", ";

        outfile2 << "      check_res" << fold_num << "     <= '0';\n";

        outfile3 << "IF( fold_indx = \"" << binary(fold_num, fold_factor - 1) << "\" ) THEN\n";
        outfile3 << "\t        IF( is_deg_odd = TRUE AND ( in_bit2 = '0' OR in_bit2 = '1' ) ) THEN\n";
        outfile3 << "\t          check_res" << fold_num << " <= check_res" << fold_num << " XOR in_bit1 XOR in_bit2;\n";
        outfile3 << "\t        ELSE\n";
        outfile3 << "\t          check_res" << fold_num << " <= check_res" << fold_num << " XOR in_bit1;\n";
        outfile3 << "\t        END IF;\n";
        outfile3 << "\t      ELS";

        outfile4 << "IF( fold_indx = \"" << binary(fold_num, fold_factor - 1) << "\" ) THEN\n";
        outfile4 << "          out_res1 <= check_res" << fold_num << ";\n";
        outfile4 << "          out_res2 <= check_res" << fold_num << ";\n";
        outfile4 << "        ELS";
    }

    outfile << "check_res" << fold_factor - 1;

    outfile2 <<"      check_res" << fold_factor - 1 << "     <= '0';\n";

    outfile3 << "IF( fold_indx = \"" << binary(fold_factor - 1, fold_factor - 1) << "\" ) THEN\n";
    outfile3 << "\t        IF( is_deg_odd = TRUE AND ( in_bit2 = '0' OR in_bit2 = '1' ) ) THEN\n";
    outfile3 << "\t          check_res" << fold_factor - 1 << " <= check_res" << fold_factor - 1 << " XOR in_bit1 XOR in_bit2;\n";
    outfile3 << "\t        ELSE\n";
    outfile3 << "\t          check_res" << fold_factor - 1 << " <= check_res" << fold_factor - 1 << " XOR in_bit1;\n";
    outfile3 << "\t        END IF;\n";

    outfile4 << "IF( fold_indx = \"" << binary(fold_factor - 1, fold_factor - 1)  << "\" ) THEN\n";
    outfile4 << "          out_res1 <= check_res" << fold_factor - 1  << ";\n";
    outfile4 << "          out_res2 <= check_res" << fold_factor - 1  << ";\n";

    outfile4.close();
    outfile3.close();
    outfile2.close();
    outfile.close();

    outfile.open(outputfile);
    outfile.close();

    append ("HDL_partitioned_files/Node_Modules/pm1.vhd", outputfile);
    append ("Generated_Files/Portions/Node_Modules/pm2.vhd", outputfile);
    append ("HDL_partitioned_files/Node_Modules/pm3.vhd", outputfile);
    append ("Generated_Files/Portions/Node_Modules/pm4.vhd", outputfile);
    append ("HDL_partitioned_files/Node_Modules/pm5.vhd", outputfile);
    append ("Generated_Files/Portions/Node_Modules/pm6.vhd", outputfile);
    append ("HDL_partitioned_files/Node_Modules/pm7.vhd", outputfile);
    append ("Generated_Files/Portions/Node_Modules/pm8.vhd", outputfile);
    append ("HDL_partitioned_files/Node_Modules/pm9.vhd", outputfile);

    return;
}

