#include <iostream>
#include <fstream>
#include <cmath>
#include <string.h>
#include <vector>
#include "C_Functions/auxiliary_functions.h"

using namespace std;

/* This function creates the Package.vhd file that contain
   vector signals, whose definitions are to be created runtime.
*/
void make_package(int nodes, int order, int fold_factor, int h_edges, int p_edges){
    ofstream outfile("HDL_Final_Modules/package.vhd");

    outfile << "LIBRARY IEEE;\n";
    outfile << "USE IEEE.STD_LOGIC_1164.ALL;\n\n";

    outfile << "PACKAGE fold_package IS\n\n";

    int ram_locations = fold_factor * ((order + (order%2)));

    outfile << "\tTYPE wr_addr_list             IS ARRAY ( 0 to " << ram_locations - 1 << " ) OF INTEGER;\n";
    outfile << "\tTYPE bus_instr_in             IS ARRAY ( 0 to " << nodes - 1 << " ) OF STD_LOGIC_VECTOR( 8 DOWNTO 0 );\n";
    outfile << "\tTYPE bus_fold_instr_in        IS ARRAY ( 0 to " << (nodes/fold_factor) - 1 << " ) OF STD_LOGIC_VECTOR( 8 DOWNTO 0 );\n";
    outfile << "\tTYPE def_network              IS ARRAY ( " << ( (nodes/fold_factor) * p_edges) - 1 << " DOWNTO 0 ) OF STD_LOGIC;\n";
    outfile << "\tTYPE x_network                IS ARRAY ( " <<( (nodes/fold_factor) * h_edges) - 1 << " DOWNTO 0 ) OF STD_LOGIC;\n";

    int pattern_bits = bin_digits( ((order + (order%2))/2) - 1);
    int fold_bits = bin_digits( fold_factor - 1);
    int microcode_bits = 31 + 2*fold_bits + 2*pattern_bits ;

    outfile << "\tSUBTYPE out_bus             IS STD_LOGIC_VECTOR( " << nodes - 1 << " DOWNTO 0 );\n";
    outfile << "\tSUBTYPE bus_instr_vec         IS STD_LOGIC_VECTOR( 8 DOWNTO 0 );\n";
    outfile << "\tSUBTYPE fold_indx             IS STD_LOGIC_VECTOR( " << fold_bits - 1 << " DOWNTO 0 );\n";
    outfile << "\tSUBTYPE node_indx             IS STD_LOGIC_VECTOR( " << bin_digits( (nodes/fold_factor) - 1) - 1  << " DOWNTO 0 );\n";
    outfile << "\tSUBTYPE pat_indx             IS STD_LOGIC_VECTOR( " << pattern_bits - 1 << " DOWNTO 0 );\n";
    outfile << "\tSUBTYPE microcode_bus        IS STD_LOGIC_VECTOR( " << microcode_bits - 1 << " DOWNTO 0 );\n";
    outfile << "\tSUBTYPE mem_addr_bus        IS STD_LOGIC_VECTOR( " << bin_digits(ram_locations - 1) - 1 << " DOWNTO 0 );\n";
    outfile << "\tSUBTYPE rom_sel_bus            IS STD_LOGIC_VECTOR( " << bin_digits( (nodes/fold_factor) - 1 ) - 1 << " DOWNTO 0 );\n";
    outfile << "\tSUBTYPE addr_bus             IS STD_LOGIC_VECTOR( " << bin_digits(microcode_bits - 1) - 1 << " DOWNTO 0 );\n";
    outfile << "\tSUBTYPE rxvd_fold_bus         IS STD_LOGIC_VECTOR( " << (nodes/fold_factor) - 1  << " DOWNTO 0 );\n";
    outfile << "\tSUBTYPE procinpbus            IS STD_LOGIC_VECTOR( " << (nodes/fold_factor) - 1  << " DOWNTO 0 );\n";

    string is_deg_odd = (order%2 == 1) ? "TRUE":"FALSE";

    outfile << "\tCONSTANT no_unfolded_pap   : INTEGER     := " << (order + (order%2) )/2 << ";\n";
    outfile << "\tCONSTANT fold_factor     : INTEGER     := " << fold_factor << ";\n";
    outfile << "\tCONSTANT is_deg_odd      : BOOLEAN     := " << is_deg_odd << ";\n\n";
    outfile << "END PACKAGE fold_package;\n";

    outfile.close();
    return;
}

