/*
 * Author: Utkarsh Gupta
 * Created on: June 5, 2012
 * File Name : make_switch_module.cpp
 * Project Name : Automation of Folding (DDP)
 * Description : This file creates the Switch Module VHDL Files
 *            i.e. the PPU Switch Module File and the PMU Switch
 *            Module File along with all its Componenet Module Files
 */

#include <iostream>
#include <fstream>
#include <cmath>
#include <string.h>
#include <vector>
#include "C_Functions/auxiliary_functions.h"

using std::vector;
using namespace std;

void make_switch_module(int folded_nodes, int num_edges, int switch_type, const char* outputfile)
{
    string q = "";
    for(int j = 0; j<bin_digits(folded_nodes); j++)
    {
      q += "Z";
    }

    ofstream outfile("Generated_Files/Portions/Switch/switch_end.vhd");

    outfile << "\n\nPROCESS( sel1 )\n";
    outfile << "BEGIN\n";
    outfile << "\tIF( To_StdULogicVector(sel1) /= \"" << q << "\" ) THEN\n";
    outfile << "\t\tIF( ( CONV_INTEGER(sel1) + CONV_INTEGER(fu_id) ) < " << folded_nodes << " ) THEN\n";
    outfile << "\t\t\tlocal_sel1 <= ( CONV_INTEGER(sel1) + CONV_INTEGER(fu_id) );\n";
    outfile << "\t\tELSE \n\t\t\tlocal_sel1 <=( ( CONV_INTEGER(sel1) + CONV_INTEGER(fu_id) ) - " << folded_nodes << " );\n";
    outfile << "\t\tEND IF;\n";
    outfile << "\tELSE\n";
    outfile << "\t\tlocal_sel1 <= -1; \n";
    outfile << "\tEND IF;\n";
    outfile << "END PROCESS;\n\n";
    outfile << "PROCESS( sel2 )\n";
    outfile << "BEGIN\n";
    outfile << "\tIF( To_StdULogicVector(sel2) /= \"" << q << "\" ) THEN\n";
    outfile << "\t\tIF( ( CONV_INTEGER(sel2) + CONV_INTEGER(fu_id) ) < " << folded_nodes << " ) THEN\n";
    outfile << "\t\t\tlocal_sel2 <= ( CONV_INTEGER(sel2) + CONV_INTEGER(fu_id) );\n";
    outfile << "\t\tELSE \n\t\t\tlocal_sel2 <=( ( CONV_INTEGER(sel2) + CONV_INTEGER(fu_id) ) - " << folded_nodes << " );\n";
    outfile << "\t\tEND IF;\n";
    outfile << "\tELSE\n";
    outfile << "\t\tlocal_sel2 <= -1;\n";
    outfile << "\tEND IF;\n";
    outfile << "END PROCESS;\n\n";
    outfile << "END ARCHITECTURE behav;\n";

    outfile.close();

    if ( switch_type % 2 == 0 ){
        ofstream outfile1("Generated_Files/Portions/Switch/switch_1.vhd");
        ofstream outfile2("Generated_Files/Portions/Switch/switch_2.vhd");

        outfile1 << "\n";
        for ( int edge_no = 0; edge_no < num_edges; edge_no++)
        {
            outfile1 <<"           in_val_"<< edge_no + 1 <<"      : IN STD_LOGIC;\n";
            outfile2 <<" in_val_"<< edge_no + 1 <<",";
        }

        outfile2 << " out_val_1, out_val_2 );";
        outfile2.close();
        outfile1.close();
    }
    else {
        ofstream outfile1("Generated_Files/Portions/Switch/switch_1.vhd");
        ofstream outfile2("Generated_Files/Portions/Switch/switch_2.vhd");

        outfile1 << "\n";
        for ( int edge_no = 0; edge_no < num_edges-1; edge_no++)
        {
            outfile1 <<"           out_val_"<< edge_no + 1 <<"     : OUT STD_LOGIC;\n";
            outfile2 <<" out_val_"<< edge_no + 1 <<",";
        }

            outfile1 <<"           out_val_"<< num_edges <<"     : OUT STD_LOGIC\n\t\t);\n";
            outfile2 <<" out_val_"<< num_edges <<");";

        outfile2.close();
        outfile1.close();
    }

    outfile.open("Generated_Files/Portions/Switch/switch_3.vhd");
    outfile << "SIGNAL local_sel1, local_sel2: INTEGER range -1 to "<< folded_nodes + 2 << ";\n";
    outfile.close();

    outfile.open(outputfile);
    outfile.close();

    switch (switch_type){
        case 0:
            append("HDL_partitioned_files/Switch/hps_start.vhd", outputfile);
            append("Generated_Files/Portions/Switch/switch_1.vhd", outputfile);
            append("HDL_partitioned_files/Switch/hps_mid1.vhd", outputfile);
            append("Generated_Files/Portions/Switch/switch_1.vhd", outputfile);
            append("HDL_partitioned_files/Switch/hps_mid2.vhd", outputfile);
            append("Generated_Files/Portions/Switch/switch_3.vhd", outputfile);
            append("HDL_partitioned_files/Switch/hps_mid3.vhd", outputfile);
            break;
        case 1:
            append("HDL_partitioned_files/Switch/hms_start.vhd", outputfile);
            append("Generated_Files/Portions/Switch/switch_1.vhd", outputfile);
            append("HDL_partitioned_files/Switch/hms_mid1.vhd", outputfile);
            append("Generated_Files/Portions/Switch/switch_1.vhd", outputfile);
            append("HDL_partitioned_files/Switch/hms_mid2.vhd", outputfile);
            append("Generated_Files/Portions/Switch/switch_3.vhd", outputfile);
            append("HDL_partitioned_files/Switch/hms_mid3.vhd", outputfile);
            break;
        case 2:
            append("HDL_partitioned_files/Switch/pps_start.vhd", outputfile);
            append("Generated_Files/Portions/Switch/switch_1.vhd", outputfile);
            append("HDL_partitioned_files/Switch/pps_mid1.vhd", outputfile);
            append("Generated_Files/Portions/Switch/switch_1.vhd", outputfile);
            append("HDL_partitioned_files/Switch/pps_mid2.vhd", outputfile);
            append("Generated_Files/Portions/Switch/switch_3.vhd", outputfile);
            append("HDL_partitioned_files/Switch/pps_mid3.vhd", outputfile);
            break;
        case 3:
            append("HDL_partitioned_files/Switch/pms_start.vhd", outputfile);
            append("Generated_Files/Portions/Switch/switch_1.vhd", outputfile);
            append("HDL_partitioned_files/Switch/pms_mid1.vhd", outputfile);
            append("Generated_Files/Portions/Switch/switch_1.vhd", outputfile);
            append("HDL_partitioned_files/Switch/pms_mid2.vhd", outputfile);
            append("Generated_Files/Portions/Switch/switch_3.vhd", outputfile);
            append("HDL_partitioned_files/Switch/pms_mid3.vhd", outputfile);
            break;
    }
    append("Generated_Files/Portions/Switch/switch_2.vhd", outputfile);
    append("Generated_Files/Portions/Switch/switch_end.vhd", outputfile);
}

void make_demux_module( int edge_conn[], int num_edges, int folded_nodes, int side_num, const char* outputfile)
{
    ofstream outfile("Generated_Files/Portions/Demux/demux_2.vhd");
    ofstream outfile_2("Generated_Files/Portions/Demux/demux_4.vhd");
    if ( side_num == 0)
        outfile << "h_demux IS\n";
    else
        outfile << "p_demux IS\n";

    outfile << "\tPORT (  clk          : IN STD_LOGIC;\n";
    outfile << "\t\t\tline_sel1    : IN INTEGER;\n";
    outfile << "\t\t\tline_sel2    : IN INTEGER;\n";
    outfile << "\t\t\tdemux_id     : IN node_indx;\n";
    outfile << "\t\t\ten_demux     : IN STD_LOGIC;\n";
    outfile << "\t\t\tin_val1      : IN STD_LOGIC;\n";
    outfile << "\t\t\tin_val2      : IN STD_LOGIC;\n";

    for ( int edge_no = 0; edge_no < num_edges-1; edge_no++)
    {
        outfile <<"\t\t\tout_val_"<<edge_no+1<<"     : OUT STD_LOGIC;\n";
        outfile_2 <<"\t\t\tout_val_"<<edge_no+1<<" <= 'Z';\n";
    }
    outfile <<"\t\t\tout_val_"<< num_edges <<"     : OUT STD_LOGIC\n\t\t);\nEND ";
    outfile_2 <<"\t\t\tout_val_"<< num_edges <<" <= 'Z';\n";

    outfile_2.close();
    outfile.close();

    outfile.open("Generated_Files/Portions/Demux/demux_5.vhd");
    outfile_2.open("Generated_Files/Portions/Demux/demux_6.vhd");

    outfile << "\t\t";
    outfile_2 << "\t\t";
    for (int node_sel = 0; node_sel < folded_nodes - 1; node_sel++)
    {
        outfile << "IF( line_sel1 = " << node_sel <<" ) THEN\n\t\t\t";
        outfile_2 << "IF( line_sel2 = " << node_sel <<" ) THEN\n\t\t\t";
        for (int demux_id = 0; demux_id < folded_nodes; demux_id++)
        {
            if ( select_line( edge_conn, num_edges,  demux_id, folded_nodes, node_sel, 1) > -1 ) {
                outfile << "IF( demux_id = \"" << binary( demux_id, folded_nodes - 1) << "\") THEN\n";
                outfile << "\t\t\t\tout_val_" << select_line( edge_conn, num_edges,  demux_id, folded_nodes, node_sel, 1) + 1 <<" <=    in_val1;\n";
                outfile << "\t\t\t\t -- No access Defined --;\n";
                outfile << "\t\t\tELS";
            }

            if ( select_line( edge_conn, num_edges,  demux_id, folded_nodes, node_sel, 2) > -1) {
                outfile_2 << "IF( demux_id = \"" << binary( demux_id, folded_nodes - 1) << "\") THEN\n";
                outfile_2 << "\t\t\t\tout_val_" << select_line( edge_conn, num_edges,  demux_id, folded_nodes, node_sel, 2) + 1 <<" <=    in_val2;\n";
                outfile_2 << "\t\t\t\t -- No access Defined --;\n";
                outfile_2 << "\t\t\tELS";
            }
        }
        outfile << "E\n\t\t\t\tout_val_1 <= 'Z';\n";
        outfile << "\t\t\tEND IF;\n";
        outfile << "\t\tELS";

        outfile_2 << "E\n\t\t\t\tout_val_2 <= 'Z';\n";
        outfile_2 << "\t\t\tEND IF;\n";
        outfile_2 << "\t\tELS";
    }

    outfile << "IF( line_sel1 = " << folded_nodes - 1 <<" ) THEN\n\t\t\t";
    outfile_2 << "IF( line_sel2 = " << folded_nodes - 1 <<" ) THEN\n\t\t\t";
    for (int demux_id = 0; demux_id < folded_nodes; demux_id++)
    {
        if ( select_line( edge_conn, num_edges,  demux_id, folded_nodes, folded_nodes - 1, 1) > -1) {
            outfile << "IF( demux_id = \"" << binary( demux_id, folded_nodes - 1) << "\") THEN\n";
            outfile << "\t\t\t\tout_val_" << select_line( edge_conn, num_edges,  demux_id, folded_nodes, folded_nodes - 1, 1) + 1 <<" <=    in_val1;\n";
            outfile << "\t\t\t\t -- No access Defined --;\n";
            outfile << "\t\t\tELS";
        }

        if (select_line( edge_conn, num_edges,  demux_id, folded_nodes, folded_nodes - 1, 2) > -1){
            outfile_2 << "IF( demux_id = \"" << binary( demux_id, folded_nodes - 1) << "\") THEN\n";
            outfile_2 << "\t\t\t\tout_val_" << select_line( edge_conn, num_edges,  demux_id, folded_nodes, folded_nodes - 1, 2) + 1 <<" <=    in_val2;\n";
            outfile_2 << "\t\t\t\t -- No access Defined --;\n";
            outfile_2 << "\t\t\tELS";
        }
    }
    outfile << "E\n\t\t\t\tout_val_1 <= 'Z';\n";
    outfile << "\t\t\tEND IF;\n";

    outfile_2 << "E\n\t\t\t\tout_val_2 <= 'Z';\n";
    outfile_2 << "\t\t\tEND IF;\n";

    outfile << "\t\tEND IF;\n\n";

    outfile_2 << "\t\tEND IF;\n\n\tELSE\n";

    outfile_2.close();
    outfile.close();

    outfile.open(outputfile);
    outfile.close();

    append( "HDL_partitioned_files/Demux/demux_1.vhd", outputfile );
    append( "Generated_Files/Portions/Demux/demux_2.vhd", outputfile );
    if ( side_num == 0)
        append( "HDL_partitioned_files/Demux/demux_3_h.vhd", outputfile );
    else
        append( "HDL_partitioned_files/Demux/demux_3_p.vhd", outputfile );

    append( "Generated_Files/Portions/Demux/demux_4.vhd", outputfile );
    append( "Generated_Files/Portions/Demux/demux_5.vhd", outputfile );
    append( "Generated_Files/Portions/Demux/demux_6.vhd", outputfile );
    append( "Generated_Files/Portions/Demux/demux_4.vhd", outputfile );
    append( "HDL_partitioned_files/Demux/demux_7.vhd", outputfile );
}

void make_mux_module( int edge_conn[], int num_edges, int folded_nodes, int side_num, const char* outputfile)
{
    ofstream outfile("Generated_Files/Portions/Mux/mux_2.vhd");

    if ( side_num == 0)
        outfile << "h_mux IS\n";
    else
        outfile << "p_mux IS\n";

    outfile << "\tPORT (   clk          : IN STD_LOGIC;\n";
    outfile << "\t\t\tline_sel1     : IN INTEGER;\n";
    outfile << "\t\t\tline_sel2     : IN INTEGER;\n";
    outfile << "\t\t\tmux_id        : IN node_indx;\n";
    outfile << "\t\t\ten_mux        : IN STD_LOGIC;\n";

    for ( int edge_no = 0; edge_no < num_edges; edge_no++)
    {
        outfile <<"\t\t\tin_val_"<<edge_no+1<<"      : IN STD_LOGIC;\n";
    }

    outfile << "\t\t\tout_val_1     : OUT STD_LOGIC;\n";
    outfile << "\t\t\tout_val_2     : OUT STD_LOGIC\n";
    outfile << "\t\t);\n";
    outfile << "END";

    outfile.close();

    outfile.open("Generated_Files/Portions/Mux/mux_4.vhd");
    ofstream outfile2("Generated_Files/Portions/Mux/mux_5.vhd");

    outfile << "\t\t";
    outfile2 << "\t\t";
    for (int node_sel = 0; node_sel < folded_nodes; node_sel++)
    {
        outfile << "IF( line_sel1 = " << node_sel <<" ) THEN\n\t\t\t";
        outfile2 << "IF( line_sel2 = " << node_sel <<" ) THEN\n\t\t";
        for (int mux_id = 0; mux_id < folded_nodes; mux_id++)
        {
            if ( select_line( edge_conn, num_edges,  mux_id, folded_nodes, node_sel, 1) > -1 ){
                outfile << "IF( mux_id = \"" << binary( mux_id, folded_nodes - 1) << "\") THEN\n";
                outfile << "\t\t\t\tout_val_1 <=    in_val_" << select_line( edge_conn, num_edges,  mux_id, folded_nodes, node_sel, 1) + 1 << ";\n";
                outfile << "\t\t\t\t -- No access Defined --;\n";
                outfile << "\t\t\tELS";
            }

            if ( select_line( edge_conn, num_edges,  mux_id, folded_nodes, node_sel, 2) > -1 ){
                outfile2 << "IF( mux_id = \"" << binary( mux_id, folded_nodes - 1) << "\") THEN\n";
                outfile2 << "\t\t\t\tout_val_2 <=    in_val_" << select_line( edge_conn, num_edges,  mux_id, folded_nodes, node_sel, 2) + 1 << ";\n";
                outfile2 << "\t\t\t\t -- No access Defined --;\n";
                outfile2 << "\t\t\tELS";
            }
        }
        outfile << "E\n\t\t\t\tout_val_1 <= 'Z';\n";
        outfile << "\t\t\tEND IF;\n";
        outfile << "\t\tELS";

        outfile2 << "E\n\t\t\t\tout_val_2 <= 'Z';\n";
        outfile2 << "\t\t\tEND IF;\n";
        outfile2 << "\t\tELS";
    }

    outfile << "E\n\t\t\tout_val_1 <= 'Z';\n";
    outfile << "\t\tEND IF;\n\n";

    outfile2 << "E\n\t\t\tout_val_2 <= 'Z';\n";
    outfile2 << "\t\tEND IF;\n";


    outfile2.close();
    outfile.close();

    outfile.open(outputfile);
    outfile.close();

    append( "HDL_partitioned_files/Mux/mux_1.vhd", outputfile );
    append( "Generated_Files/Portions/Mux/mux_2.vhd", outputfile );
    if ( side_num == 0)
        append( "HDL_partitioned_files/Mux/mux_3_h.vhd", outputfile );
    else
        append( "HDL_partitioned_files/Mux/mux_3_p.vhd", outputfile );

    append( "Generated_Files/Portions/Mux/mux_4.vhd", outputfile );
    append( "Generated_Files/Portions/Mux/mux_5.vhd", outputfile );
    append( "HDL_partitioned_files/Mux/mux_6.vhd", outputfile );
}

void make_rom_module(int conn[], int order, int nodes_folded, int rom_type, const char* outputfile)
{
    int pattern = (order%2 == 0) ? (order/2) : ((order/2)+1);
    string q = "";

    // Preparing default driver value if pattern id is illegal.
    for(int j = 0; j<bin_digits(nodes_folded - 1 ); j++)
    {
      q += "Z";
    }

    ofstream outfile("Generated_Files/Portions/ROMs/rom_2.vhd");
    outfile << "    -- Output first line selection/line address\n";
    outfile << "    CASE pat_id IS\n";

    // Preparing listing of port selection values, to be emitted at out1 port.
    for(int i = 0; i<pattern; i++)
    {
        string y = (conn[2*i] != -1) ? (binary((conn[2*i]%nodes_folded), nodes_folded - 1 )) : q;
        outfile << "        WHEN \""<< binary(i,pattern - 1) <<"\"   => out1 <= \""<< y <<"\"; -- "<<conn[2*i]<<"\n";
    }
    outfile << "        WHEN OTHERS => out1 <= \""<< q <<"\";\n    END CASE;\n\n";

    // Preparing listing of port selection values, to be emitted at out2 port.
    outfile << "    -- Output second line selection/line address\n";
    outfile << "    CASE pat_id IS\n";
    for(int i = 0; i<pattern-1; i++)
    {
        string y = (conn[2*i+1] != -1) ? (binary((conn[2*i+1]%nodes_folded), nodes_folded - 1 )) : q;
        outfile << "        WHEN \""<< binary(i,pattern - 1) <<"\"   => out2 <= \""<< y <<"\";-- "<<conn[2*i+1]<<"\n";
    }
    string x = (order%2 == 0) ? (binary(conn[2*pattern-1]%nodes_folded, nodes_folded)) : q;
    outfile << "        WHEN \""<< binary(pattern - 1, pattern - 1) <<"\"   => out2 <= \""<<x<<"\";\n";
    outfile << "        WHEN OTHERS => out1 <= \""<<q<<"\";\n    END CASE;\n\n";
    outfile.close();

    outfile.open(outputfile);
    outfile.close();

    switch (rom_type) {
        case 0:
            append("HDL_partitioned_files/ROMs/rom_hps1.vhd",outputfile);
            break;
        case 1:
            append("HDL_partitioned_files/ROMs/rom_hms1.vhd",outputfile);
            break;
        case 2:
            append("HDL_partitioned_files/ROMs/rom_pps1.vhd",outputfile);
            break;
        case 3:
            append("HDL_partitioned_files/ROMs/rom_pms1.vhd",outputfile);
            break;
    }
    append("Generated_Files/Portions/ROMs/rom_2.vhd",outputfile);
    append("HDL_partitioned_files/ROMs/rom_3.vhd",outputfile);
}

