/*
 * Author: Utkarsh Gupta
 * Created on: June 5, 2012
 * File Name : write_vhdl_modules.cpp
 * Project Name : Automation of Folding (DDP)
 * Description : This function uses the parameters
 *            from the Parameter Struct and writes the
 *            VHDL files for all the modules
 */

#include <iostream>
#include <fstream>
#include <cmath>
#include <string.h>
#include <vector>
#include "C_Functions/data_structs.h"
#include "C_Functions/auxiliary_functions.h"
#include "C_Functions/Refinement_4/fold_PG.h"
#include "C_Functions/Refinement_4/make_node_module.h"
#include "C_Functions/Refinement_4/make_mem_module.h"
#include "C_Functions/Refinement_4/make_switch_module.h"
#include "C_Functions/Refinement_4/make_package.h"
#include "C_Functions/Refinement_4/make_seq.h"
#include "C_Functions/Refinement_4/make_ctrl_str.h"

using namespace std;
using std::vector;

void write_vhdl_modules(parameters PG_parameters)
{
    cout << "\n\nREFINEMENT 4 - WRITING THE VHDL MODULES \n";
    string default_addrZ = "";
    string default_addr0 = "";
    int ram_locations = (PG_parameters.fold_factor * (PG_parameters.order + (PG_parameters.order%2)));

    // Preparing default address value if clock is low.
    for(int j = 0; j<bin_digits(ram_locations); j++)
    {
      default_addrZ += "Z";
      default_addr0 += "0";
    }

    int *inv_hyp_conn = new int[PG_parameters.order]; // Connection array for HYPERPLANE NODE#1
    int *inv_poi_conn = new int[PG_parameters.order]; // Connection array for POINT NODE#1

    for(int node = 0; node < PG_parameters.order; node++){
        inv_hyp_conn[node] =  (PG_parameters.nodes - PG_parameters.hyp_conn[ node ]) % PG_parameters.nodes;
        inv_poi_conn[node] =  (PG_parameters.nodes - PG_parameters.point_conn[ node ]) % PG_parameters.nodes;
    }

    make_fold_pg( PG_parameters, bin_digits(ram_locations), "HDL_Final_Modules/fold_PG.vhd");
    cout << "\n1\n";
    make_tdp_mem(PG_parameters.order, PG_parameters.fold_factor, "HDL_Final_Modules/tdp_mem.vhd");
    cout << "\n2\n";
    make_add_mux(PG_parameters.order, PG_parameters.fold_factor, default_addrZ, "HDL_Final_Modules/add_mux.vhd");
    cout << "\n3\n";
    make_rd_addrgen(PG_parameters.order, PG_parameters.fold_factor, default_addrZ, default_addr0, bin_digits(ram_locations), "HDL_Final_Modules/addrgen.vhd");
    cout << "\n4\n";
    make_wr_addrgen(PG_parameters.order, PG_parameters.nodes, PG_parameters.fold_factor, PG_parameters.hyp_conn, default_addrZ, bin_digits(ram_locations), "HDL_Final_Modules/hyp_wr_addrgen.vhd", "HDL_Final_Modules/point_wr_addrgen.vhd");
    cout << "\n5\n";
    make_mem_module( "HDL_Final_Modules/hyp_mem_unit.vhd", "HDL_Final_Modules/point_mem_unit.vhd");
    cout << "\n6\n";

    int *edge_conn1 = new int[PG_parameters.num_h_edges];    // Array Pointer : FOLDED HYPERPLANE #0
                                                            //                    Connections
    int *edge_conn2 = new int[PG_parameters.num_p_edges];    // Array Pointer : FOLDED POINT #0
                                                            //                    Connections
    int edge_num = 0;
    int edge_num2 = 0;
    for(int i = 0; i < PG_parameters.folded_nodes; i++){
        for(int j = 0; j < PG_parameters.h0_to_p[i]; j++){
            edge_conn1[edge_num] = i;
            edge_num++;
        }

        for(int k = 0; k < PG_parameters.p0_to_h[i]; k++){
            edge_conn2[edge_num2] = i;
            edge_num2++;
        }
    }

    make_mux_module( edge_conn1 , PG_parameters.num_h_edges, PG_parameters.folded_nodes, 0, "HDL_Final_Modules/ic_h_mux.vhd");
    cout << "\n7\n";
    make_mux_module( edge_conn2 , PG_parameters.num_p_edges, PG_parameters.folded_nodes, 1, "HDL_Final_Modules/ic_p_mux.vhd");
    cout << "\n8\n";
    make_demux_module( edge_conn2 , PG_parameters.num_p_edges, PG_parameters.folded_nodes, 0, "HDL_Final_Modules/ic_h_demux.vhd");
    cout << "\n9\n";
    make_demux_module( edge_conn1 , PG_parameters.num_h_edges, PG_parameters.folded_nodes, 1, "HDL_Final_Modules/ic_p_demux.vhd");
    cout << "\n10\n";
    make_rom_module(PG_parameters.hyp_conn, PG_parameters.order, PG_parameters.folded_nodes, 0, "HDL_Final_Modules/ic_rom_hps.vhd");
    cout << "\n11\n";
    make_rom_module(inv_poi_conn, PG_parameters.order, PG_parameters.folded_nodes, 1, "HDL_Final_Modules/ic_rom_hms.vhd");
    cout << "\n12\n";

    make_rom_module(PG_parameters.point_conn, PG_parameters.order, PG_parameters.folded_nodes, 2, "HDL_Final_Modules/ic_rom_pps.vhd");
    cout << "\n13\n";
    make_rom_module(inv_hyp_conn, PG_parameters.order, PG_parameters.folded_nodes, 3, "HDL_Final_Modules/ic_rom_pms.vhd");
    cout << "\n14\n";
    make_switch_module(PG_parameters.folded_nodes, PG_parameters.num_h_edges, 0, "HDL_Final_Modules/ic_hps.vhd");
    cout << "\n15\n";
    make_switch_module(PG_parameters.folded_nodes, PG_parameters.num_p_edges, 1, "HDL_Final_Modules/ic_hms.vhd");
    cout << "\n16\n";
    make_switch_module(PG_parameters.folded_nodes, PG_parameters.num_p_edges, 2, "HDL_Final_Modules/ic_pps.vhd");
    cout << "\n17\n";
    make_switch_module(PG_parameters.folded_nodes, PG_parameters.num_h_edges, 3, "HDL_Final_Modules/ic_pms.vhd");
    cout << "\n18\n";
    make_hyp_module( PG_parameters.fold_factor, "HDL_Final_Modules/ic_hyp_node.vhd");
    cout << "\n19\n";
    make_point_module( PG_parameters.fold_factor, "HDL_Final_Modules/ic_point_node.vhd");
    cout << "\n20\n";
    make_package(PG_parameters.nodes, PG_parameters.order, PG_parameters.fold_factor, PG_parameters.num_h_edges, PG_parameters.num_p_edges);
    cout << "\n21\n";
    make_seq(PG_parameters.order, PG_parameters.fold_factor);
    cout << "\n22\n";
    make_ctrl_str(PG_parameters.order, PG_parameters.fold_factor);
    cout << "\n23\n";
    return;
}

