/*
 * Author: Utkarsh Gupta
 * Created on: June 5, 2012
 * File Name : data_structs.h
 * Project Name : Automation of Folding (DDP)
 * Description : This file contains the definations of the
 *             structs and classes used in various refinements
 *             and for storing the parameters for the Bipartite
 *             PG Graph
 */

    // Refinement 2 : Initial PPU Struct with $rho$ Input Ports and 2 output ports.
    struct PPU{
        int id; // PPU Index

        int *ip_ports; // $rho$ Input Ports
        int num_ips; // Number of Inputs

        int op_ports[2]; // 2 Output Ports
    };

    // Refinement 2 : Initial PMU Struct with $rho$ Output Ports and 2 input ports.
    struct PMU{
        int id; // PMU Index

        int ip_ports[2]; // 2 Input Ports

        int *op_ports; // $rho$ Output Ports
        int num_ops; // Number of outputs
    };

    /* Refinement 2 : Initial wire Struct.
     A wire is connected between the HYPERPLANE Node and POINT Node
             from a PMU port to a PPU port
     Since we have 2 instances of PPUs and PMUs (for H and P),
            we will make 2 instances of the wire network (H->P and P->H)*/
    struct wire{
        int id;

        int data; /* This struct will also be used in the Refinement 3
                 and thus must carry the Data from the Hyperplane
                 to the Point Side or vice versa*/

        /* Since one instance of the wire is connected between the
        a PPU and a PMU.
        We store the ID and PORT NUMBER for the PPU and PMU
        */
        int PMU_id;
        int PMU_port_num;

        int PPU_id;
        int PPU_port_num;
    };

    /* This structure contains the parameters of the Bipartite PG
    that are defined till the Refinement 2
    */
    struct parameters{
        int nodes; // Total number of PG Nodes
        int order; // Order of each node (Hyp and Point)
                   // The order of both sides are equal
                   // as the number of nodes on both sides are equal

        int *hyp_conn;    // Array Pointer : UNFOLDED HYPERPLANE #0
                        //                    Connections
        int *point_conn;    // Array Pointer : UNFOLDED POINT #0
                            //                    Connections

        int fold_factor;    // Fold Factor
        int folded_nodes;    // Nodes in the FOLDED GRAPH
        int num_dummy;    // Number of Dummy Nodes added in case of
                        // Prime number of nodes

        int num_h_edges; // In the FOLDED PG GRAPH, the number of edges from the
                         // HYPERPLANE # 0 to Point nodes
        int num_p_edges; // In the FOLDED PG GRAPH, the number of edges from the
                         // POINT # 0 to Hyperplane nodes

        int *h0_to_p;       // H->P connections in a FOLDED PG GRAPH
                           // Array Index signifies the FOLDED POINT NODE INDEX the H0 is connected to
                           // The value in the array element signifies the *number of connections* (0/1/2)
                           // between Hyperplane Node 0 and Folded POINT NODE Index (array index)

        int *p0_to_h;       // P->H connections in a FOLDED PG GRAPH
                           // Array Index signifies the FOLDED HYPERPLANE NODE INDEX the P0 is connected to
                           // The value in the array element signifies the *number of connections* (0/1/2)
                           // between Point Node 0 and Folded HYPERPLANE NODE Index (array index)

        PPU *H_PPU;    // Array of Hyperplane PPU structures
        PPU *P_PPU;    // Array of Point PPU structures

        PMU *H_PMU;    // Array of Hyperplane PMU Structures
        PMU *P_PMU;    // Array of Point PMU Structures

        wire *h_to_p_wires;    // Array of wires from H->P
        wire *p_to_h_wires;    // Array of wires from P->H
    };






