LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE WORK.fold_package.ALL;

ENTITY fold_pg_decoder IS
    PORT( clk           : IN STD_LOGIC;
          en_decode     : IN STD_LOGIC;
          intr_in       : IN bus_instr_in;
          valid_cword   : OUT STD_LOGIC;
          no_iter       : OUT INTEGER;
          output        : OUT out_bus
        );
END ENTITY fold_pg_decoder;

ARCHITECTURE Behavioral OF fold_pg_decoder IS

---------------------bit node component-----------------------

COMPONENT bit_f_unit
    PORT( clk            : IN STD_LOGIC;
          in_bit1        : IN STD_LOGIC;
          in_bit2        : IN STD_LOGIC;
          intrinsic_vec  : IN bus_instr_vec;
          fold_indx      : IN fold_indx;
          en_acc         : IN STD_LOGIC;
          en_flip        : IN STD_LOGIC;
          en_first_time  : IN STD_LOGIC;
          en_part_store  : IN STD_LOGIC;
          en_count_reset : IN STD_LOGIC;
          en_out         : IN STD_LOGIC;
          bit_estimate   : OUT STD_LOGIC;
          out_bit1       : OUT STD_LOGIC;
          out_bit2       : OUT STD_LOGIC
        );
END COMPONENT;

--------------------check node component----------------------

COMPONENT check_f_unit
    PORT( clk           : IN STD_LOGIC;
          rst_c         : IN STD_LOGIC;
          in_bit1       : IN STD_LOGIC;
          in_bit2       : IN STD_LOGIC;
          en_check      : IN STD_LOGIC;
          en_out        : IN STD_LOGIC;
          fold_indx     : IN fold_indx;
          out_res1      : OUT STD_LOGIC;
          out_res2      : OUT STD_LOGIC
        );
END COMPONENT;

--------------------memory unit component---------------------

COMPONENT hyp_mem_unit
    PORT ( clk       : IN STD_LOGIC;
           rst       : IN STD_LOGIC;
           en_use    : IN STD_LOGIC;
           rd_w_bar1 : IN STD_LOGIC;
           rd_w_bar2 : IN STD_LOGIC;
           mu_id     : IN node_indx;
           d_in1     : IN STD_LOGIC;
           d_in2     : IN STD_LOGIC;
           d_out1    : OUT STD_LOGIC;
           d_out2    : OUT STD_LOGIC
         );
END COMPONENT;

COMPONENT point_mem_unit
    PORT ( clk       : IN STD_LOGIC;
           rst       : IN STD_LOGIC;
           en_use    : IN STD_LOGIC;
           rd_w_bar1 : IN STD_LOGIC;
           rd_w_bar2 : IN STD_LOGIC;
           mu_id     : IN node_indx;
           d_in1     : IN STD_LOGIC;
           d_in2     : IN STD_LOGIC;
           d_out1    : OUT STD_LOGIC;
           d_out2    : OUT STD_LOGIC
         );
END COMPONENT;
-------------------memory switch components-------------------

COMPONENT h_m_switch
    PORT ( clk          : IN STD_LOGIC;
           en_m_switch  : IN STD_LOGIC;
           pat_id       : IN pat_indx;
           fu_id        : IN node_indx;
           in_val1      : IN STD_LOGIC;
           in_val2      : IN STD_LOGIC;
           out_val_1     : OUT STD_LOGIC;
           out_val_2     : OUT STD_LOGIC;
           out_val_3     : OUT STD_LOGIC;
           out_val_4     : OUT STD_LOGIC;
           out_val_5     : OUT STD_LOGIC;
           out_val_6     : OUT STD_LOGIC;
           out_val_7     : OUT STD_LOGIC;
           out_val_8     : OUT STD_LOGIC
         );

END COMPONENT;

COMPONENT p_m_switch
    PORT ( clk          : IN STD_LOGIC;
           en_m_switch  : IN STD_LOGIC;
           pat_id       : IN pat_indx;
           fu_id        : IN node_indx;
           in_val1      : IN STD_LOGIC;
           in_val2      : IN STD_LOGIC;
           out_val_1     : OUT STD_LOGIC;
           out_val_2     : OUT STD_LOGIC;
           out_val_3     : OUT STD_LOGIC;
           out_val_4     : OUT STD_LOGIC;
           out_val_5     : OUT STD_LOGIC;
           out_val_6     : OUT STD_LOGIC;
           out_val_7     : OUT STD_LOGIC;
           out_val_8     : OUT STD_LOGIC
         );

END COMPONENT;

-------------------processor switch components-------------------

COMPONENT h_p_switch
    PORT ( clk          : IN STD_LOGIC;
           en_p_switch  : IN STD_LOGIC;
           pat_id       : IN pat_indx;
           fu_id        : IN node_indx;
           in_val_1         : IN STD_LOGIC;
           in_val_2         : IN STD_LOGIC;
           in_val_3         : IN STD_LOGIC;
           in_val_4         : IN STD_LOGIC;
           in_val_5         : IN STD_LOGIC;
           in_val_6         : IN STD_LOGIC;
           in_val_7         : IN STD_LOGIC;
           in_val_8         : IN STD_LOGIC;
           out_val_1        : OUT STD_LOGIC;
           out_val_2        : OUT STD_LOGIC
         );
END COMPONENT;

COMPONENT p_p_switch
    PORT ( clk          : IN STD_LOGIC;
           en_p_switch  : IN STD_LOGIC;
           pat_id       : IN pat_indx;
           fu_id        : IN node_indx;
           in_val_1         : IN STD_LOGIC;
           in_val_2         : IN STD_LOGIC;
           in_val_3         : IN STD_LOGIC;
           in_val_4         : IN STD_LOGIC;
           in_val_5         : IN STD_LOGIC;
           in_val_6         : IN STD_LOGIC;
           in_val_7         : IN STD_LOGIC;
           in_val_8         : IN STD_LOGIC;
           out_val_1        : OUT STD_LOGIC;
           out_val_2        : OUT STD_LOGIC
         );
END COMPONENT;

------------------control store component---------------------

COMPONENT control_store
    PORT( addr  : IN addr_bus;
          dout  : OUT microcode_bus
        );
END COMPONENT;

-------------Sequence generator for microcodes----------------

COMPONENT sequencer
    PORT( clock  : IN STD_LOGIC;
          en_seq : IN STD_LOGIC;
          add    : OUT addr_bus
        );
END COMPONENT;

FOR ALL:bit_f_unit      USE ENTITY WORK.bit_f_unit( behav );
FOR ALL:check_f_unit    USE ENTITY WORK.check_f_unit( behav );
FOR ALL:hyp_mem_unit    USE ENTITY WORK.hyp_mem_unit( behav );
FOR ALL:point_mem_unit  USE ENTITY WORK.point_mem_unit( behav );
FOR ALL:h_p_switch      USE ENTITY WORK.h_p_switch( behav );
FOR ALL:p_p_switch      USE ENTITY WORK.p_p_switch( behav );
FOR ALL:h_m_switch      USE ENTITY WORK.h_m_switch( behav );
FOR ALL:p_m_switch      USE ENTITY WORK.p_m_switch( behav );
FOR ALL:control_store   USE ENTITY WORK.control_store( behav );
FOR ALL:sequencer       USE ENTITY WORK.sequencer( behav );

------------------memory signals------------
-- Signals related to memories for bit nodes

SIGNAL en_bmem_use      : STD_LOGIC;
SIGNAL rst_b            : STD_LOGIC;
SIGNAL en_bmem_rd_wbar1 : STD_LOGIC;
SIGNAL en_bmem_rd_wbar2 : STD_LOGIC;
SIGNAL en_m_switch_b    : STD_LOGIC;
SIGNAL en_p_switch_b    : STD_LOGIC;
SIGNAL pat_id_b         : pat_indx;

SIGNAL bmu_in1           : procinpbus;
SIGNAL bmu_in2           : procinpbus;
SIGNAL bmu_out1          : procinpbus;
SIGNAL bmu_out2          : procinpbus;

-- Signals related to memories for check nodes

SIGNAL en_cmem_use      : STD_LOGIC;
SIGNAL rst_c            : STD_LOGIC;
SIGNAL en_cmem_rd_wbar1 : STD_LOGIC;
SIGNAL en_cmem_rd_wbar2 : STD_LOGIC;
SIGNAL en_m_switch_c    : STD_LOGIC;
SIGNAL en_p_switch_c    : STD_LOGIC;
SIGNAL pat_id_c         : pat_indx;
SIGNAL fold_intr_in     : bus_fold_instr_in;

SIGNAL cmu_in1           : procinpbus;
SIGNAL cmu_in2           : procinpbus;
SIGNAL cmu_out1          : procinpbus;
SIGNAL cmu_out2          : procinpbus;

------------------validity check signal----------------
SIGNAL num_of_iter        : INTEGER;

------------------interconnect network------------------
SIGNAL b_to_c_wires       : def_network;
SIGNAL c_to_b_wires       : x_network;

------------------mmux signals--------------------------
SIGNAL en_mmux_b          : STD_LOGIC;
SIGNAL en_mmux_c          : STD_LOGIC;

------------------pmux signals--------------------------
SIGNAL en_pmux_b          : STD_LOGIC;
SIGNAL en_pmux_c          : STD_LOGIC;

------------------bit node signals----------------------
SIGNAL fold_indx_b      : fold_indx;
SIGNAL en_acc_b         : STD_LOGIC;
SIGNAL en_flip_b        : STD_LOGIC;
SIGNAL en_first_time_b  : STD_LOGIC;
SIGNAL en_part_store_b  : STD_LOGIC;
SIGNAL en_count_reset_b : STD_LOGIC;
SIGNAL en_out_b         : STD_LOGIC;

SIGNAL bfu_in1          : procinpbus;
SIGNAL bfu_in2          : procinpbus;
SIGNAL unused_b         : procinpbus;


-------------------check node signals-------------------
SIGNAL en_check_c   : STD_LOGIC;
SIGNAL en_out_c     : STD_LOGIC;
SIGNAL fold_indx_c  : fold_indx;

SIGNAL cfu_in1          : procinpbus;
SIGNAL cfu_in2          : procinpbus;
SIGNAL unused_c         : procinpbus;

---------Control microcodes sequencing signals---------
SIGNAL en_mem             : STD_LOGIC;
SIGNAL en_sequencing      : STD_LOGIC;
SIGNAL microcode          : microcode_bus;
SIGNAL ctrl_store_addr    : addr_bus;

SIGNAL valid_word         : STD_LOGIC;
SIGNAL decide_cword       : STD_LOGIC;
SIGNAL rxvd_code          : out_bus;
SIGNAL pcheck_out         : out_bus;
SIGNAL rxvd_fold          : rxvd_fold_bus;

--------------------------------------------------------------------------
BEGIN
-- Generate all the functional and memory units
G1:FOR i IN 18 DOWNTO 0 GENERATE


    bfu :  bit_f_unit       PORT MAP( clk, bfu_in1(i), bfu_in2(i),
                                      fold_intr_in(i),
                                      fold_indx_b, en_acc_b, en_flip_b,
                                      en_first_time_b, en_part_store_b,
                                      en_count_reset_b, en_out_b,
                                      rxvd_fold(i), bmu_in1(i), bmu_in2(i)
                                    );


    cfu :  check_f_unit     PORT MAP( clk, rst_c, cfu_in1(i), cfu_in2(i),
                                      en_check_c,
                                      en_out_c, fold_indx_c,
                                      cmu_in1(i), cmu_in2(i)
                                    );


    h_mem  :  hyp_mem_unit    PORT MAP( clk, rst_b, en_bmem_use,
                                      en_bmem_rd_wbar1, en_bmem_rd_wbar2,
                                      CONV_STD_LOGIC_VECTOR( i, 5 ),
                                      bmu_in1(i), bmu_in2(i),
                                      bmu_out1(i), bmu_out2(i)
                                    );

    p_mem:  point_mem_unit    PORT MAP( clk, rst_c, en_cmem_use,
                                      en_cmem_rd_wbar1, en_cmem_rd_wbar2,
                                      CONV_STD_LOGIC_VECTOR( i, 5 ),
                                      cmu_in1(i), cmu_in2(i),
                                      cmu_out1(i), cmu_out2(i)
                                    );
END GENERATE G1;

-- Microcode sequencing components.
cntr_addgen :   sequencer      PORT MAP (clk, en_sequencing, ctrl_store_addr);
cntr_store  :   control_store  PORT MAP ( ctrl_store_addr, microcode );

h_p_switch_0:  h_p_switch PORT MAP( clk, en_p_switch_b, pat_id_b,
                                "00000", -- 0th unit
                                c_to_b_wires( 0 ),
                                c_to_b_wires( 1 ),
                                c_to_b_wires( 2 ),
                                c_to_b_wires( 3 ),
                                c_to_b_wires( 4 ),
                                c_to_b_wires( 5 ),
                                c_to_b_wires( 6 ),
                                c_to_b_wires( 7 ),
                                bfu_in1( 0 ), bfu_in2( 0 )
                                  );

h_p_switch_1:  h_p_switch PORT MAP( clk, en_p_switch_b, pat_id_b,
                                "00001", -- 1th unit
                                c_to_b_wires( 8 ),
                                c_to_b_wires( 9 ),
                                c_to_b_wires( 10 ),
                                c_to_b_wires( 11 ),
                                c_to_b_wires( 12 ),
                                c_to_b_wires( 13 ),
                                c_to_b_wires( 14 ),
                                c_to_b_wires( 15 ),
                                bfu_in1( 1 ), bfu_in2( 1 )
                                  );

h_p_switch_2:  h_p_switch PORT MAP( clk, en_p_switch_b, pat_id_b,
                                "00010", -- 2th unit
                                c_to_b_wires( 16 ),
                                c_to_b_wires( 17 ),
                                c_to_b_wires( 18 ),
                                c_to_b_wires( 19 ),
                                c_to_b_wires( 20 ),
                                c_to_b_wires( 21 ),
                                c_to_b_wires( 22 ),
                                c_to_b_wires( 23 ),
                                bfu_in1( 2 ), bfu_in2( 2 )
                                  );

h_p_switch_3:  h_p_switch PORT MAP( clk, en_p_switch_b, pat_id_b,
                                "00011", -- 3th unit
                                c_to_b_wires( 24 ),
                                c_to_b_wires( 25 ),
                                c_to_b_wires( 26 ),
                                c_to_b_wires( 27 ),
                                c_to_b_wires( 28 ),
                                c_to_b_wires( 29 ),
                                c_to_b_wires( 30 ),
                                c_to_b_wires( 31 ),
                                bfu_in1( 3 ), bfu_in2( 3 )
                                  );

h_p_switch_4:  h_p_switch PORT MAP( clk, en_p_switch_b, pat_id_b,
                                "00100", -- 4th unit
                                c_to_b_wires( 32 ),
                                c_to_b_wires( 33 ),
                                c_to_b_wires( 34 ),
                                c_to_b_wires( 35 ),
                                c_to_b_wires( 36 ),
                                c_to_b_wires( 37 ),
                                c_to_b_wires( 38 ),
                                c_to_b_wires( 39 ),
                                bfu_in1( 4 ), bfu_in2( 4 )
                                  );

h_p_switch_5:  h_p_switch PORT MAP( clk, en_p_switch_b, pat_id_b,
                                "00101", -- 5th unit
                                c_to_b_wires( 40 ),
                                c_to_b_wires( 41 ),
                                c_to_b_wires( 42 ),
                                c_to_b_wires( 43 ),
                                c_to_b_wires( 44 ),
                                c_to_b_wires( 45 ),
                                c_to_b_wires( 46 ),
                                c_to_b_wires( 47 ),
                                bfu_in1( 5 ), bfu_in2( 5 )
                                  );

h_p_switch_6:  h_p_switch PORT MAP( clk, en_p_switch_b, pat_id_b,
                                "00110", -- 6th unit
                                c_to_b_wires( 48 ),
                                c_to_b_wires( 49 ),
                                c_to_b_wires( 50 ),
                                c_to_b_wires( 51 ),
                                c_to_b_wires( 52 ),
                                c_to_b_wires( 53 ),
                                c_to_b_wires( 54 ),
                                c_to_b_wires( 55 ),
                                bfu_in1( 6 ), bfu_in2( 6 )
                                  );

h_p_switch_7:  h_p_switch PORT MAP( clk, en_p_switch_b, pat_id_b,
                                "00111", -- 7th unit
                                c_to_b_wires( 56 ),
                                c_to_b_wires( 57 ),
                                c_to_b_wires( 58 ),
                                c_to_b_wires( 59 ),
                                c_to_b_wires( 60 ),
                                c_to_b_wires( 61 ),
                                c_to_b_wires( 62 ),
                                c_to_b_wires( 63 ),
                                bfu_in1( 7 ), bfu_in2( 7 )
                                  );

h_p_switch_8:  h_p_switch PORT MAP( clk, en_p_switch_b, pat_id_b,
                                "01000", -- 8th unit
                                c_to_b_wires( 64 ),
                                c_to_b_wires( 65 ),
                                c_to_b_wires( 66 ),
                                c_to_b_wires( 67 ),
                                c_to_b_wires( 68 ),
                                c_to_b_wires( 69 ),
                                c_to_b_wires( 70 ),
                                c_to_b_wires( 71 ),
                                bfu_in1( 8 ), bfu_in2( 8 )
                                  );

h_p_switch_9:  h_p_switch PORT MAP( clk, en_p_switch_b, pat_id_b,
                                "01001", -- 9th unit
                                c_to_b_wires( 72 ),
                                c_to_b_wires( 73 ),
                                c_to_b_wires( 74 ),
                                c_to_b_wires( 75 ),
                                c_to_b_wires( 76 ),
                                c_to_b_wires( 77 ),
                                c_to_b_wires( 78 ),
                                c_to_b_wires( 79 ),
                                bfu_in1( 9 ), bfu_in2( 9 )
                                  );

h_p_switch_10:  h_p_switch PORT MAP( clk, en_p_switch_b, pat_id_b,
                                "01010", -- 10th unit
                                c_to_b_wires( 80 ),
                                c_to_b_wires( 81 ),
                                c_to_b_wires( 82 ),
                                c_to_b_wires( 83 ),
                                c_to_b_wires( 84 ),
                                c_to_b_wires( 85 ),
                                c_to_b_wires( 86 ),
                                c_to_b_wires( 87 ),
                                bfu_in1( 10 ), bfu_in2( 10 )
                                  );

h_p_switch_11:  h_p_switch PORT MAP( clk, en_p_switch_b, pat_id_b,
                                "01011", -- 11th unit
                                c_to_b_wires( 88 ),
                                c_to_b_wires( 89 ),
                                c_to_b_wires( 90 ),
                                c_to_b_wires( 91 ),
                                c_to_b_wires( 92 ),
                                c_to_b_wires( 93 ),
                                c_to_b_wires( 94 ),
                                c_to_b_wires( 95 ),
                                bfu_in1( 11 ), bfu_in2( 11 )
                                  );

h_p_switch_12:  h_p_switch PORT MAP( clk, en_p_switch_b, pat_id_b,
                                "01100", -- 12th unit
                                c_to_b_wires( 96 ),
                                c_to_b_wires( 97 ),
                                c_to_b_wires( 98 ),
                                c_to_b_wires( 99 ),
                                c_to_b_wires( 100 ),
                                c_to_b_wires( 101 ),
                                c_to_b_wires( 102 ),
                                c_to_b_wires( 103 ),
                                bfu_in1( 12 ), bfu_in2( 12 )
                                  );

h_p_switch_13:  h_p_switch PORT MAP( clk, en_p_switch_b, pat_id_b,
                                "01101", -- 13th unit
                                c_to_b_wires( 104 ),
                                c_to_b_wires( 105 ),
                                c_to_b_wires( 106 ),
                                c_to_b_wires( 107 ),
                                c_to_b_wires( 108 ),
                                c_to_b_wires( 109 ),
                                c_to_b_wires( 110 ),
                                c_to_b_wires( 111 ),
                                bfu_in1( 13 ), bfu_in2( 13 )
                                  );

h_p_switch_14:  h_p_switch PORT MAP( clk, en_p_switch_b, pat_id_b,
                                "01110", -- 14th unit
                                c_to_b_wires( 112 ),
                                c_to_b_wires( 113 ),
                                c_to_b_wires( 114 ),
                                c_to_b_wires( 115 ),
                                c_to_b_wires( 116 ),
                                c_to_b_wires( 117 ),
                                c_to_b_wires( 118 ),
                                c_to_b_wires( 119 ),
                                bfu_in1( 14 ), bfu_in2( 14 )
                                  );

h_p_switch_15:  h_p_switch PORT MAP( clk, en_p_switch_b, pat_id_b,
                                "01111", -- 15th unit
                                c_to_b_wires( 120 ),
                                c_to_b_wires( 121 ),
                                c_to_b_wires( 122 ),
                                c_to_b_wires( 123 ),
                                c_to_b_wires( 124 ),
                                c_to_b_wires( 125 ),
                                c_to_b_wires( 126 ),
                                c_to_b_wires( 127 ),
                                bfu_in1( 15 ), bfu_in2( 15 )
                                  );

h_p_switch_16:  h_p_switch PORT MAP( clk, en_p_switch_b, pat_id_b,
                                "10000", -- 16th unit
                                c_to_b_wires( 128 ),
                                c_to_b_wires( 129 ),
                                c_to_b_wires( 130 ),
                                c_to_b_wires( 131 ),
                                c_to_b_wires( 132 ),
                                c_to_b_wires( 133 ),
                                c_to_b_wires( 134 ),
                                c_to_b_wires( 135 ),
                                bfu_in1( 16 ), bfu_in2( 16 )
                                  );

h_p_switch_17:  h_p_switch PORT MAP( clk, en_p_switch_b, pat_id_b,
                                "10001", -- 17th unit
                                c_to_b_wires( 136 ),
                                c_to_b_wires( 137 ),
                                c_to_b_wires( 138 ),
                                c_to_b_wires( 139 ),
                                c_to_b_wires( 140 ),
                                c_to_b_wires( 141 ),
                                c_to_b_wires( 142 ),
                                c_to_b_wires( 143 ),
                                bfu_in1( 17 ), bfu_in2( 17 )
                                  );

h_p_switch_18:  h_p_switch PORT MAP( clk, en_p_switch_b, pat_id_b,
                                "10010", -- 18th unit
                                c_to_b_wires( 144 ),
                                c_to_b_wires( 145 ),
                                c_to_b_wires( 146 ),
                                c_to_b_wires( 147 ),
                                c_to_b_wires( 148 ),
                                c_to_b_wires( 149 ),
                                c_to_b_wires( 150 ),
                                c_to_b_wires( 151 ),
                                bfu_in1( 18 ), bfu_in2( 18 )
                                  );

h_m_switch_0:  h_m_switch   PORT MAP( clk, en_m_switch_b, pat_id_b,
                                "00000", -- 0th unit
                                bmu_out1(0), bmu_out2(0),
                                b_to_c_wires( 0 ),
                                b_to_c_wires( 8 ),
                                b_to_c_wires( 16 ),
                                b_to_c_wires( 32 ),
                                b_to_c_wires( 40 ),
                                b_to_c_wires( 64 ),
                                b_to_c_wires( 80 ),
                                b_to_c_wires( 112 )
                              );
h_m_switch_1:  h_m_switch   PORT MAP( clk, en_m_switch_b, pat_id_b,
                                "00001", -- 1th unit
                                bmu_out1(1), bmu_out2(1),
                                b_to_c_wires( 9 ),
                                b_to_c_wires( 17 ),
                                b_to_c_wires( 24 ),
                                b_to_c_wires( 41 ),
                                b_to_c_wires( 48 ),
                                b_to_c_wires( 72 ),
                                b_to_c_wires( 88 ),
                                b_to_c_wires( 120 )
                              );
h_m_switch_2:  h_m_switch   PORT MAP( clk, en_m_switch_b, pat_id_b,
                                "00010", -- 2th unit
                                bmu_out1(2), bmu_out2(2),
                                b_to_c_wires( 18 ),
                                b_to_c_wires( 25 ),
                                b_to_c_wires( 33 ),
                                b_to_c_wires( 49 ),
                                b_to_c_wires( 56 ),
                                b_to_c_wires( 81 ),
                                b_to_c_wires( 96 ),
                                b_to_c_wires( 128 )
                              );
h_m_switch_3:  h_m_switch   PORT MAP( clk, en_m_switch_b, pat_id_b,
                                "00011", -- 3th unit
                                bmu_out1(3), bmu_out2(3),
                                b_to_c_wires( 26 ),
                                b_to_c_wires( 34 ),
                                b_to_c_wires( 42 ),
                                b_to_c_wires( 57 ),
                                b_to_c_wires( 65 ),
                                b_to_c_wires( 89 ),
                                b_to_c_wires( 104 ),
                                b_to_c_wires( 136 )
                              );
h_m_switch_4:  h_m_switch   PORT MAP( clk, en_m_switch_b, pat_id_b,
                                "00100", -- 4th unit
                                bmu_out1(4), bmu_out2(4),
                                b_to_c_wires( 35 ),
                                b_to_c_wires( 43 ),
                                b_to_c_wires( 50 ),
                                b_to_c_wires( 66 ),
                                b_to_c_wires( 73 ),
                                b_to_c_wires( 97 ),
                                b_to_c_wires( 113 ),
                                b_to_c_wires( 144 )
                              );
h_m_switch_5:  h_m_switch   PORT MAP( clk, en_m_switch_b, pat_id_b,
                                "00101", -- 5th unit
                                bmu_out1(5), bmu_out2(5),
                                b_to_c_wires( 1 ),
                                b_to_c_wires( 44 ),
                                b_to_c_wires( 51 ),
                                b_to_c_wires( 58 ),
                                b_to_c_wires( 74 ),
                                b_to_c_wires( 82 ),
                                b_to_c_wires( 105 ),
                                b_to_c_wires( 121 )
                              );
h_m_switch_6:  h_m_switch   PORT MAP( clk, en_m_switch_b, pat_id_b,
                                "00110", -- 6th unit
                                bmu_out1(6), bmu_out2(6),
                                b_to_c_wires( 10 ),
                                b_to_c_wires( 52 ),
                                b_to_c_wires( 59 ),
                                b_to_c_wires( 67 ),
                                b_to_c_wires( 83 ),
                                b_to_c_wires( 90 ),
                                b_to_c_wires( 114 ),
                                b_to_c_wires( 129 )
                              );
h_m_switch_7:  h_m_switch   PORT MAP( clk, en_m_switch_b, pat_id_b,
                                "00111", -- 7th unit
                                bmu_out1(7), bmu_out2(7),
                                b_to_c_wires( 19 ),
                                b_to_c_wires( 60 ),
                                b_to_c_wires( 68 ),
                                b_to_c_wires( 75 ),
                                b_to_c_wires( 91 ),
                                b_to_c_wires( 98 ),
                                b_to_c_wires( 122 ),
                                b_to_c_wires( 137 )
                              );
h_m_switch_8:  h_m_switch   PORT MAP( clk, en_m_switch_b, pat_id_b,
                                "01000", -- 8th unit
                                bmu_out1(8), bmu_out2(8),
                                b_to_c_wires( 27 ),
                                b_to_c_wires( 69 ),
                                b_to_c_wires( 76 ),
                                b_to_c_wires( 84 ),
                                b_to_c_wires( 99 ),
                                b_to_c_wires( 106 ),
                                b_to_c_wires( 130 ),
                                b_to_c_wires( 145 )
                              );
h_m_switch_9:  h_m_switch   PORT MAP( clk, en_m_switch_b, pat_id_b,
                                "01001", -- 9th unit
                                bmu_out1(9), bmu_out2(9),
                                b_to_c_wires( 2 ),
                                b_to_c_wires( 36 ),
                                b_to_c_wires( 77 ),
                                b_to_c_wires( 85 ),
                                b_to_c_wires( 92 ),
                                b_to_c_wires( 107 ),
                                b_to_c_wires( 115 ),
                                b_to_c_wires( 138 )
                              );
h_m_switch_10:  h_m_switch   PORT MAP( clk, en_m_switch_b, pat_id_b,
                                "01010", -- 10th unit
                                bmu_out1(10), bmu_out2(10),
                                b_to_c_wires( 11 ),
                                b_to_c_wires( 45 ),
                                b_to_c_wires( 86 ),
                                b_to_c_wires( 93 ),
                                b_to_c_wires( 100 ),
                                b_to_c_wires( 116 ),
                                b_to_c_wires( 123 ),
                                b_to_c_wires( 146 )
                              );
h_m_switch_11:  h_m_switch   PORT MAP( clk, en_m_switch_b, pat_id_b,
                                "01011", -- 11th unit
                                bmu_out1(11), bmu_out2(11),
                                b_to_c_wires( 3 ),
                                b_to_c_wires( 20 ),
                                b_to_c_wires( 53 ),
                                b_to_c_wires( 94 ),
                                b_to_c_wires( 101 ),
                                b_to_c_wires( 108 ),
                                b_to_c_wires( 124 ),
                                b_to_c_wires( 131 )
                              );
h_m_switch_12:  h_m_switch   PORT MAP( clk, en_m_switch_b, pat_id_b,
                                "01100", -- 12th unit
                                bmu_out1(12), bmu_out2(12),
                                b_to_c_wires( 12 ),
                                b_to_c_wires( 28 ),
                                b_to_c_wires( 61 ),
                                b_to_c_wires( 102 ),
                                b_to_c_wires( 109 ),
                                b_to_c_wires( 117 ),
                                b_to_c_wires( 132 ),
                                b_to_c_wires( 139 )
                              );
h_m_switch_13:  h_m_switch   PORT MAP( clk, en_m_switch_b, pat_id_b,
                                "01101", -- 13th unit
                                bmu_out1(13), bmu_out2(13),
                                b_to_c_wires( 21 ),
                                b_to_c_wires( 37 ),
                                b_to_c_wires( 70 ),
                                b_to_c_wires( 110 ),
                                b_to_c_wires( 118 ),
                                b_to_c_wires( 125 ),
                                b_to_c_wires( 140 ),
                                b_to_c_wires( 147 )
                              );
h_m_switch_14:  h_m_switch   PORT MAP( clk, en_m_switch_b, pat_id_b,
                                "01110", -- 14th unit
                                bmu_out1(14), bmu_out2(14),
                                b_to_c_wires( 4 ),
                                b_to_c_wires( 29 ),
                                b_to_c_wires( 46 ),
                                b_to_c_wires( 78 ),
                                b_to_c_wires( 119 ),
                                b_to_c_wires( 126 ),
                                b_to_c_wires( 133 ),
                                b_to_c_wires( 148 )
                              );
h_m_switch_15:  h_m_switch   PORT MAP( clk, en_m_switch_b, pat_id_b,
                                "01111", -- 15th unit
                                bmu_out1(15), bmu_out2(15),
                                b_to_c_wires( 5 ),
                                b_to_c_wires( 13 ),
                                b_to_c_wires( 38 ),
                                b_to_c_wires( 54 ),
                                b_to_c_wires( 87 ),
                                b_to_c_wires( 127 ),
                                b_to_c_wires( 134 ),
                                b_to_c_wires( 141 )
                              );
h_m_switch_16:  h_m_switch   PORT MAP( clk, en_m_switch_b, pat_id_b,
                                "10000", -- 16th unit
                                bmu_out1(16), bmu_out2(16),
                                b_to_c_wires( 14 ),
                                b_to_c_wires( 22 ),
                                b_to_c_wires( 47 ),
                                b_to_c_wires( 62 ),
                                b_to_c_wires( 95 ),
                                b_to_c_wires( 135 ),
                                b_to_c_wires( 142 ),
                                b_to_c_wires( 149 )
                              );
h_m_switch_17:  h_m_switch   PORT MAP( clk, en_m_switch_b, pat_id_b,
                                "10001", -- 17th unit
                                bmu_out1(17), bmu_out2(17),
                                b_to_c_wires( 6 ),
                                b_to_c_wires( 23 ),
                                b_to_c_wires( 30 ),
                                b_to_c_wires( 55 ),
                                b_to_c_wires( 71 ),
                                b_to_c_wires( 103 ),
                                b_to_c_wires( 143 ),
                                b_to_c_wires( 150 )
                              );
h_m_switch_18:  h_m_switch   PORT MAP( clk, en_m_switch_b, pat_id_b,
                                "10010", -- 18th unit
                                bmu_out1(18), bmu_out2(18),
                                b_to_c_wires( 7 ),
                                b_to_c_wires( 15 ),
                                b_to_c_wires( 31 ),
                                b_to_c_wires( 39 ),
                                b_to_c_wires( 63 ),
                                b_to_c_wires( 79 ),
                                b_to_c_wires( 111 ),
                                b_to_c_wires( 151 )
                              );
p_p_switch_0:  p_p_switch PORT MAP( clk, en_p_switch_c, pat_id_c,
                                "00000", -- 0th unit
                                b_to_c_wires( 0 ),
                                b_to_c_wires( 1 ),
                                b_to_c_wires( 2 ),
                                b_to_c_wires( 3 ),
                                b_to_c_wires( 4 ),
                                b_to_c_wires( 5 ),
                                b_to_c_wires( 6 ),
                                b_to_c_wires( 7 ),
                                cfu_in1( 0 ), cfu_in2( 0 )
                                  );

p_p_switch_1:  p_p_switch PORT MAP( clk, en_p_switch_c, pat_id_c,
                                "00001", -- 1th unit
                                b_to_c_wires( 8 ),
                                b_to_c_wires( 9 ),
                                b_to_c_wires( 10 ),
                                b_to_c_wires( 11 ),
                                b_to_c_wires( 12 ),
                                b_to_c_wires( 13 ),
                                b_to_c_wires( 14 ),
                                b_to_c_wires( 15 ),
                                cfu_in1( 1 ), cfu_in2( 1 )
                                  );

p_p_switch_2:  p_p_switch PORT MAP( clk, en_p_switch_c, pat_id_c,
                                "00010", -- 2th unit
                                b_to_c_wires( 16 ),
                                b_to_c_wires( 17 ),
                                b_to_c_wires( 18 ),
                                b_to_c_wires( 19 ),
                                b_to_c_wires( 20 ),
                                b_to_c_wires( 21 ),
                                b_to_c_wires( 22 ),
                                b_to_c_wires( 23 ),
                                cfu_in1( 2 ), cfu_in2( 2 )
                                  );

p_p_switch_3:  p_p_switch PORT MAP( clk, en_p_switch_c, pat_id_c,
                                "00011", -- 3th unit
                                b_to_c_wires( 24 ),
                                b_to_c_wires( 25 ),
                                b_to_c_wires( 26 ),
                                b_to_c_wires( 27 ),
                                b_to_c_wires( 28 ),
                                b_to_c_wires( 29 ),
                                b_to_c_wires( 30 ),
                                b_to_c_wires( 31 ),
                                cfu_in1( 3 ), cfu_in2( 3 )
                                  );

p_p_switch_4:  p_p_switch PORT MAP( clk, en_p_switch_c, pat_id_c,
                                "00100", -- 4th unit
                                b_to_c_wires( 32 ),
                                b_to_c_wires( 33 ),
                                b_to_c_wires( 34 ),
                                b_to_c_wires( 35 ),
                                b_to_c_wires( 36 ),
                                b_to_c_wires( 37 ),
                                b_to_c_wires( 38 ),
                                b_to_c_wires( 39 ),
                                cfu_in1( 4 ), cfu_in2( 4 )
                                  );

p_p_switch_5:  p_p_switch PORT MAP( clk, en_p_switch_c, pat_id_c,
                                "00101", -- 5th unit
                                b_to_c_wires( 40 ),
                                b_to_c_wires( 41 ),
                                b_to_c_wires( 42 ),
                                b_to_c_wires( 43 ),
                                b_to_c_wires( 44 ),
                                b_to_c_wires( 45 ),
                                b_to_c_wires( 46 ),
                                b_to_c_wires( 47 ),
                                cfu_in1( 5 ), cfu_in2( 5 )
                                  );

p_p_switch_6:  p_p_switch PORT MAP( clk, en_p_switch_c, pat_id_c,
                                "00110", -- 6th unit
                                b_to_c_wires( 48 ),
                                b_to_c_wires( 49 ),
                                b_to_c_wires( 50 ),
                                b_to_c_wires( 51 ),
                                b_to_c_wires( 52 ),
                                b_to_c_wires( 53 ),
                                b_to_c_wires( 54 ),
                                b_to_c_wires( 55 ),
                                cfu_in1( 6 ), cfu_in2( 6 )
                                  );

p_p_switch_7:  p_p_switch PORT MAP( clk, en_p_switch_c, pat_id_c,
                                "00111", -- 7th unit
                                b_to_c_wires( 56 ),
                                b_to_c_wires( 57 ),
                                b_to_c_wires( 58 ),
                                b_to_c_wires( 59 ),
                                b_to_c_wires( 60 ),
                                b_to_c_wires( 61 ),
                                b_to_c_wires( 62 ),
                                b_to_c_wires( 63 ),
                                cfu_in1( 7 ), cfu_in2( 7 )
                                  );

p_p_switch_8:  p_p_switch PORT MAP( clk, en_p_switch_c, pat_id_c,
                                "01000", -- 8th unit
                                b_to_c_wires( 64 ),
                                b_to_c_wires( 65 ),
                                b_to_c_wires( 66 ),
                                b_to_c_wires( 67 ),
                                b_to_c_wires( 68 ),
                                b_to_c_wires( 69 ),
                                b_to_c_wires( 70 ),
                                b_to_c_wires( 71 ),
                                cfu_in1( 8 ), cfu_in2( 8 )
                                  );

p_p_switch_9:  p_p_switch PORT MAP( clk, en_p_switch_c, pat_id_c,
                                "01001", -- 9th unit
                                b_to_c_wires( 72 ),
                                b_to_c_wires( 73 ),
                                b_to_c_wires( 74 ),
                                b_to_c_wires( 75 ),
                                b_to_c_wires( 76 ),
                                b_to_c_wires( 77 ),
                                b_to_c_wires( 78 ),
                                b_to_c_wires( 79 ),
                                cfu_in1( 9 ), cfu_in2( 9 )
                                  );

p_p_switch_10:  p_p_switch PORT MAP( clk, en_p_switch_c, pat_id_c,
                                "01010", -- 10th unit
                                b_to_c_wires( 80 ),
                                b_to_c_wires( 81 ),
                                b_to_c_wires( 82 ),
                                b_to_c_wires( 83 ),
                                b_to_c_wires( 84 ),
                                b_to_c_wires( 85 ),
                                b_to_c_wires( 86 ),
                                b_to_c_wires( 87 ),
                                cfu_in1( 10 ), cfu_in2( 10 )
                                  );

p_p_switch_11:  p_p_switch PORT MAP( clk, en_p_switch_c, pat_id_c,
                                "01011", -- 11th unit
                                b_to_c_wires( 88 ),
                                b_to_c_wires( 89 ),
                                b_to_c_wires( 90 ),
                                b_to_c_wires( 91 ),
                                b_to_c_wires( 92 ),
                                b_to_c_wires( 93 ),
                                b_to_c_wires( 94 ),
                                b_to_c_wires( 95 ),
                                cfu_in1( 11 ), cfu_in2( 11 )
                                  );

p_p_switch_12:  p_p_switch PORT MAP( clk, en_p_switch_c, pat_id_c,
                                "01100", -- 12th unit
                                b_to_c_wires( 96 ),
                                b_to_c_wires( 97 ),
                                b_to_c_wires( 98 ),
                                b_to_c_wires( 99 ),
                                b_to_c_wires( 100 ),
                                b_to_c_wires( 101 ),
                                b_to_c_wires( 102 ),
                                b_to_c_wires( 103 ),
                                cfu_in1( 12 ), cfu_in2( 12 )
                                  );

p_p_switch_13:  p_p_switch PORT MAP( clk, en_p_switch_c, pat_id_c,
                                "01101", -- 13th unit
                                b_to_c_wires( 104 ),
                                b_to_c_wires( 105 ),
                                b_to_c_wires( 106 ),
                                b_to_c_wires( 107 ),
                                b_to_c_wires( 108 ),
                                b_to_c_wires( 109 ),
                                b_to_c_wires( 110 ),
                                b_to_c_wires( 111 ),
                                cfu_in1( 13 ), cfu_in2( 13 )
                                  );

p_p_switch_14:  p_p_switch PORT MAP( clk, en_p_switch_c, pat_id_c,
                                "01110", -- 14th unit
                                b_to_c_wires( 112 ),
                                b_to_c_wires( 113 ),
                                b_to_c_wires( 114 ),
                                b_to_c_wires( 115 ),
                                b_to_c_wires( 116 ),
                                b_to_c_wires( 117 ),
                                b_to_c_wires( 118 ),
                                b_to_c_wires( 119 ),
                                cfu_in1( 14 ), cfu_in2( 14 )
                                  );

p_p_switch_15:  p_p_switch PORT MAP( clk, en_p_switch_c, pat_id_c,
                                "01111", -- 15th unit
                                b_to_c_wires( 120 ),
                                b_to_c_wires( 121 ),
                                b_to_c_wires( 122 ),
                                b_to_c_wires( 123 ),
                                b_to_c_wires( 124 ),
                                b_to_c_wires( 125 ),
                                b_to_c_wires( 126 ),
                                b_to_c_wires( 127 ),
                                cfu_in1( 15 ), cfu_in2( 15 )
                                  );

p_p_switch_16:  p_p_switch PORT MAP( clk, en_p_switch_c, pat_id_c,
                                "10000", -- 16th unit
                                b_to_c_wires( 128 ),
                                b_to_c_wires( 129 ),
                                b_to_c_wires( 130 ),
                                b_to_c_wires( 131 ),
                                b_to_c_wires( 132 ),
                                b_to_c_wires( 133 ),
                                b_to_c_wires( 134 ),
                                b_to_c_wires( 135 ),
                                cfu_in1( 16 ), cfu_in2( 16 )
                                  );

p_p_switch_17:  p_p_switch PORT MAP( clk, en_p_switch_c, pat_id_c,
                                "10001", -- 17th unit
                                b_to_c_wires( 136 ),
                                b_to_c_wires( 137 ),
                                b_to_c_wires( 138 ),
                                b_to_c_wires( 139 ),
                                b_to_c_wires( 140 ),
                                b_to_c_wires( 141 ),
                                b_to_c_wires( 142 ),
                                b_to_c_wires( 143 ),
                                cfu_in1( 17 ), cfu_in2( 17 )
                                  );

p_p_switch_18:  p_p_switch PORT MAP( clk, en_p_switch_c, pat_id_c,
                                "10010", -- 18th unit
                                b_to_c_wires( 144 ),
                                b_to_c_wires( 145 ),
                                b_to_c_wires( 146 ),
                                b_to_c_wires( 147 ),
                                b_to_c_wires( 148 ),
                                b_to_c_wires( 149 ),
                                b_to_c_wires( 150 ),
                                b_to_c_wires( 151 ),
                                cfu_in1( 18 ), cfu_in2( 18 )
                                  );

p_m_switch_0:  p_m_switch   PORT MAP( clk, en_m_switch_c, pat_id_c,
                                "00000", -- 0th unit
                                cmu_out1(0), cmu_out2(0),
                                c_to_b_wires( 0 ),
                                c_to_b_wires( 40 ),
                                c_to_b_wires( 72 ),
                                c_to_b_wires( 88 ),
                                c_to_b_wires( 112 ),
                                c_to_b_wires( 120 ),
                                c_to_b_wires( 136 ),
                                c_to_b_wires( 144 )
                              );
p_m_switch_1:  p_m_switch   PORT MAP( clk, en_m_switch_c, pat_id_c,
                                "00001", -- 1th unit
                                cmu_out1(1), cmu_out2(1),
                                c_to_b_wires( 1 ),
                                c_to_b_wires( 8 ),
                                c_to_b_wires( 48 ),
                                c_to_b_wires( 80 ),
                                c_to_b_wires( 96 ),
                                c_to_b_wires( 121 ),
                                c_to_b_wires( 128 ),
                                c_to_b_wires( 145 )
                              );
p_m_switch_2:  p_m_switch   PORT MAP( clk, en_m_switch_c, pat_id_c,
                                "00010", -- 2th unit
                                cmu_out1(2), cmu_out2(2),
                                c_to_b_wires( 2 ),
                                c_to_b_wires( 9 ),
                                c_to_b_wires( 16 ),
                                c_to_b_wires( 56 ),
                                c_to_b_wires( 89 ),
                                c_to_b_wires( 104 ),
                                c_to_b_wires( 129 ),
                                c_to_b_wires( 137 )
                              );
p_m_switch_3:  p_m_switch   PORT MAP( clk, en_m_switch_c, pat_id_c,
                                "00011", -- 3th unit
                                cmu_out1(3), cmu_out2(3),
                                c_to_b_wires( 10 ),
                                c_to_b_wires( 17 ),
                                c_to_b_wires( 24 ),
                                c_to_b_wires( 64 ),
                                c_to_b_wires( 97 ),
                                c_to_b_wires( 113 ),
                                c_to_b_wires( 138 ),
                                c_to_b_wires( 146 )
                              );
p_m_switch_4:  p_m_switch   PORT MAP( clk, en_m_switch_c, pat_id_c,
                                "00100", -- 4th unit
                                cmu_out1(4), cmu_out2(4),
                                c_to_b_wires( 3 ),
                                c_to_b_wires( 18 ),
                                c_to_b_wires( 25 ),
                                c_to_b_wires( 32 ),
                                c_to_b_wires( 73 ),
                                c_to_b_wires( 105 ),
                                c_to_b_wires( 122 ),
                                c_to_b_wires( 147 )
                              );
p_m_switch_5:  p_m_switch   PORT MAP( clk, en_m_switch_c, pat_id_c,
                                "00101", -- 5th unit
                                cmu_out1(5), cmu_out2(5),
                                c_to_b_wires( 4 ),
                                c_to_b_wires( 11 ),
                                c_to_b_wires( 26 ),
                                c_to_b_wires( 33 ),
                                c_to_b_wires( 41 ),
                                c_to_b_wires( 81 ),
                                c_to_b_wires( 114 ),
                                c_to_b_wires( 130 )
                              );
p_m_switch_6:  p_m_switch   PORT MAP( clk, en_m_switch_c, pat_id_c,
                                "00110", -- 6th unit
                                cmu_out1(6), cmu_out2(6),
                                c_to_b_wires( 12 ),
                                c_to_b_wires( 19 ),
                                c_to_b_wires( 34 ),
                                c_to_b_wires( 42 ),
                                c_to_b_wires( 49 ),
                                c_to_b_wires( 90 ),
                                c_to_b_wires( 123 ),
                                c_to_b_wires( 139 )
                              );
p_m_switch_7:  p_m_switch   PORT MAP( clk, en_m_switch_c, pat_id_c,
                                "00111", -- 7th unit
                                cmu_out1(7), cmu_out2(7),
                                c_to_b_wires( 20 ),
                                c_to_b_wires( 27 ),
                                c_to_b_wires( 43 ),
                                c_to_b_wires( 50 ),
                                c_to_b_wires( 57 ),
                                c_to_b_wires( 98 ),
                                c_to_b_wires( 131 ),
                                c_to_b_wires( 148 )
                              );
p_m_switch_8:  p_m_switch   PORT MAP( clk, en_m_switch_c, pat_id_c,
                                "01000", -- 8th unit
                                cmu_out1(8), cmu_out2(8),
                                c_to_b_wires( 5 ),
                                c_to_b_wires( 28 ),
                                c_to_b_wires( 35 ),
                                c_to_b_wires( 51 ),
                                c_to_b_wires( 58 ),
                                c_to_b_wires( 65 ),
                                c_to_b_wires( 106 ),
                                c_to_b_wires( 140 )
                              );
p_m_switch_9:  p_m_switch   PORT MAP( clk, en_m_switch_c, pat_id_c,
                                "01001", -- 9th unit
                                cmu_out1(9), cmu_out2(9),
                                c_to_b_wires( 13 ),
                                c_to_b_wires( 36 ),
                                c_to_b_wires( 44 ),
                                c_to_b_wires( 59 ),
                                c_to_b_wires( 66 ),
                                c_to_b_wires( 74 ),
                                c_to_b_wires( 115 ),
                                c_to_b_wires( 149 )
                              );
p_m_switch_10:  p_m_switch   PORT MAP( clk, en_m_switch_c, pat_id_c,
                                "01010", -- 10th unit
                                cmu_out1(10), cmu_out2(10),
                                c_to_b_wires( 6 ),
                                c_to_b_wires( 21 ),
                                c_to_b_wires( 45 ),
                                c_to_b_wires( 52 ),
                                c_to_b_wires( 67 ),
                                c_to_b_wires( 75 ),
                                c_to_b_wires( 82 ),
                                c_to_b_wires( 124 )
                              );
p_m_switch_11:  p_m_switch   PORT MAP( clk, en_m_switch_c, pat_id_c,
                                "01011", -- 11th unit
                                cmu_out1(11), cmu_out2(11),
                                c_to_b_wires( 14 ),
                                c_to_b_wires( 29 ),
                                c_to_b_wires( 53 ),
                                c_to_b_wires( 60 ),
                                c_to_b_wires( 76 ),
                                c_to_b_wires( 83 ),
                                c_to_b_wires( 91 ),
                                c_to_b_wires( 132 )
                              );
p_m_switch_12:  p_m_switch   PORT MAP( clk, en_m_switch_c, pat_id_c,
                                "01100", -- 12th unit
                                cmu_out1(12), cmu_out2(12),
                                c_to_b_wires( 22 ),
                                c_to_b_wires( 37 ),
                                c_to_b_wires( 61 ),
                                c_to_b_wires( 68 ),
                                c_to_b_wires( 84 ),
                                c_to_b_wires( 92 ),
                                c_to_b_wires( 99 ),
                                c_to_b_wires( 141 )
                              );
p_m_switch_13:  p_m_switch   PORT MAP( clk, en_m_switch_c, pat_id_c,
                                "01101", -- 13th unit
                                cmu_out1(13), cmu_out2(13),
                                c_to_b_wires( 30 ),
                                c_to_b_wires( 46 ),
                                c_to_b_wires( 69 ),
                                c_to_b_wires( 77 ),
                                c_to_b_wires( 93 ),
                                c_to_b_wires( 100 ),
                                c_to_b_wires( 107 ),
                                c_to_b_wires( 150 )
                              );
p_m_switch_14:  p_m_switch   PORT MAP( clk, en_m_switch_c, pat_id_c,
                                "01110", -- 14th unit
                                cmu_out1(14), cmu_out2(14),
                                c_to_b_wires( 7 ),
                                c_to_b_wires( 38 ),
                                c_to_b_wires( 54 ),
                                c_to_b_wires( 78 ),
                                c_to_b_wires( 85 ),
                                c_to_b_wires( 101 ),
                                c_to_b_wires( 108 ),
                                c_to_b_wires( 116 )
                              );
p_m_switch_15:  p_m_switch   PORT MAP( clk, en_m_switch_c, pat_id_c,
                                "01111", -- 15th unit
                                cmu_out1(15), cmu_out2(15),
                                c_to_b_wires( 15 ),
                                c_to_b_wires( 47 ),
                                c_to_b_wires( 62 ),
                                c_to_b_wires( 86 ),
                                c_to_b_wires( 94 ),
                                c_to_b_wires( 109 ),
                                c_to_b_wires( 117 ),
                                c_to_b_wires( 125 )
                              );
p_m_switch_16:  p_m_switch   PORT MAP( clk, en_m_switch_c, pat_id_c,
                                "10000", -- 16th unit
                                cmu_out1(16), cmu_out2(16),
                                c_to_b_wires( 23 ),
                                c_to_b_wires( 55 ),
                                c_to_b_wires( 70 ),
                                c_to_b_wires( 95 ),
                                c_to_b_wires( 102 ),
                                c_to_b_wires( 118 ),
                                c_to_b_wires( 126 ),
                                c_to_b_wires( 133 )
                              );
p_m_switch_17:  p_m_switch   PORT MAP( clk, en_m_switch_c, pat_id_c,
                                "10001", -- 17th unit
                                cmu_out1(17), cmu_out2(17),
                                c_to_b_wires( 31 ),
                                c_to_b_wires( 63 ),
                                c_to_b_wires( 79 ),
                                c_to_b_wires( 103 ),
                                c_to_b_wires( 110 ),
                                c_to_b_wires( 127 ),
                                c_to_b_wires( 134 ),
                                c_to_b_wires( 142 )
                              );
p_m_switch_18:  p_m_switch   PORT MAP( clk, en_m_switch_c, pat_id_c,
                                "10010", -- 18th unit
                                cmu_out1(18), cmu_out2(18),
                                c_to_b_wires( 39 ),
                                c_to_b_wires( 71 ),
                                c_to_b_wires( 87 ),
                                c_to_b_wires( 111 ),
                                c_to_b_wires( 119 ),
                                c_to_b_wires( 135 ),
                                c_to_b_wires( 143 ),
                                c_to_b_wires( 151 )
                              );
decide_cword     <= microcode( 26 );

--------Control signal mappings for bit nodes ---------
en_m_switch_b    <= microcode( 24 );
en_bmem_rd_wbar1 <= microcode( 23 );
en_bmem_rd_wbar2 <= microcode( 22 );
en_bmem_use      <= microcode( 21 );
rst_b            <= microcode( 20 );
en_out_b         <= microcode( 19 );
en_flip_b        <= microcode( 18 );
en_acc_b         <= microcode( 17 );
en_part_store_b  <= microcode( 16 );
en_count_reset_b <= microcode( 15 );
en_first_time_b  <= microcode( 14 );
en_p_switch_b    <= microcode( 13 );

--------Control signal mappings for check nodes ---------
en_m_switch_c    <= microcode( 7 );
en_cmem_rd_wbar1 <= microcode( 6 );
en_cmem_rd_wbar2 <= microcode( 5 );
en_cmem_use      <= microcode( 4 );
rst_c            <= microcode( 3 );
en_out_c         <= microcode( 2 );
en_check_c       <= microcode( 1 );
en_p_switch_c    <= microcode( 0 );
pat_id_b         <= microcode( 29 downto 28 );
fold_indx_b      <= microcode( 32 downto 31 );
pat_id_c         <= microcode( 35 downto 34 );
fold_indx_c      <= microcode( 38 downto 37 );

-- Storing an estimated codeword, in case during next 'check' phase, all parity checks turn out to be satisfied.

be1: rxvd_code( 18 downto 0 ) <= rxvd_fold WHEN ( en_out_b = '1' AND fold_indx_b = "00" ) ELSE rxvd_code( 18 downto 0 );

be2: rxvd_code( 37 downto 19 ) <= rxvd_fold WHEN ( en_out_b = '1' AND fold_indx_b = "01" ) ELSE rxvd_code( 37 downto 19 );

be3: rxvd_code( 56 downto 38 ) <= rxvd_fold WHEN ( en_out_b = '1' AND fold_indx_b = "10" ) ELSE rxvd_code( 56 downto 38 );

-- Accumulating all parity check outputs, to see if the decoding has converged.

pc1: pcheck_out( 18 downto 0 ) <= cmu_in1 WHEN ( clk'EVENT AND clk = '1' AND en_out_c = '1' AND fold_indx_c = "00" ) ELSE pcheck_out( 18 downto 0 );

pc2: pcheck_out( 37 downto 19 ) <= cmu_in1 WHEN ( clk'EVENT AND clk = '1' AND en_out_c = '1' AND fold_indx_c = "01" ) ELSE pcheck_out( 37 downto 19 );

pc3: pcheck_out( 56 downto 38 ) <= cmu_in1 WHEN ( clk'EVENT AND clk = '1' AND en_out_c = '1' AND fold_indx_c = "10" ) ELSE pcheck_out( 56 downto 38 );

------------process for multiplexing intrinsic information ----------------
-- Providing the intrinsic information half a clock cycle ahead, to avoid setup time violation.
PROCESS( clk )
BEGIN
    IF( clk'EVENT AND clk = '0' ) THEN
      IF( en_first_time_b = '1' ) THEN
        IF( fold_indx_b = "00" ) THEN
         fold_intr_in( 0 ) <= intr_in( 0 );
         fold_intr_in( 1 ) <= intr_in( 1 );
         fold_intr_in( 2 ) <= intr_in( 2 );
         fold_intr_in( 3 ) <= intr_in( 3 );
         fold_intr_in( 4 ) <= intr_in( 4 );
         fold_intr_in( 5 ) <= intr_in( 5 );
         fold_intr_in( 6 ) <= intr_in( 6 );
         fold_intr_in( 7 ) <= intr_in( 7 );
         fold_intr_in( 8 ) <= intr_in( 8 );
         fold_intr_in( 9 ) <= intr_in( 9 );
         fold_intr_in( 10 ) <= intr_in( 10 );
         fold_intr_in( 11 ) <= intr_in( 11 );
         fold_intr_in( 12 ) <= intr_in( 12 );
         fold_intr_in( 13 ) <= intr_in( 13 );
         fold_intr_in( 14 ) <= intr_in( 14 );
         fold_intr_in( 15 ) <= intr_in( 15 );
         fold_intr_in( 16 ) <= intr_in( 16 );
         fold_intr_in( 17 ) <= intr_in( 17 );
         fold_intr_in( 18 ) <= intr_in( 18 );
        ELSIF( fold_indx_b = "01" ) THEN
         fold_intr_in( 0 ) <= intr_in( 19 );
         fold_intr_in( 1 ) <= intr_in( 20 );
         fold_intr_in( 2 ) <= intr_in( 21 );
         fold_intr_in( 3 ) <= intr_in( 22 );
         fold_intr_in( 4 ) <= intr_in( 23 );
         fold_intr_in( 5 ) <= intr_in( 24 );
         fold_intr_in( 6 ) <= intr_in( 25 );
         fold_intr_in( 7 ) <= intr_in( 26 );
         fold_intr_in( 8 ) <= intr_in( 27 );
         fold_intr_in( 9 ) <= intr_in( 28 );
         fold_intr_in( 10 ) <= intr_in( 29 );
         fold_intr_in( 11 ) <= intr_in( 30 );
         fold_intr_in( 12 ) <= intr_in( 31 );
         fold_intr_in( 13 ) <= intr_in( 32 );
         fold_intr_in( 14 ) <= intr_in( 33 );
         fold_intr_in( 15 ) <= intr_in( 34 );
         fold_intr_in( 16 ) <= intr_in( 35 );
         fold_intr_in( 17 ) <= intr_in( 36 );
         fold_intr_in( 18 ) <= intr_in( 37 );
        ELSIF( fold_indx_b = "10" ) THEN
         fold_intr_in( 0 ) <= intr_in( 38 );
         fold_intr_in( 1 ) <= intr_in( 39 );
         fold_intr_in( 2 ) <= intr_in( 40 );
         fold_intr_in( 3 ) <= intr_in( 41 );
         fold_intr_in( 4 ) <= intr_in( 42 );
         fold_intr_in( 5 ) <= intr_in( 43 );
         fold_intr_in( 6 ) <= intr_in( 44 );
         fold_intr_in( 7 ) <= intr_in( 45 );
         fold_intr_in( 8 ) <= intr_in( 46 );
         fold_intr_in( 9 ) <= intr_in( 47 );
         fold_intr_in( 10 ) <= intr_in( 48 );
         fold_intr_in( 11 ) <= intr_in( 49 );
         fold_intr_in( 12 ) <= intr_in( 50 );
         fold_intr_in( 13 ) <= intr_in( 51 );
         fold_intr_in( 14 ) <= intr_in( 52 );
         fold_intr_in( 15 ) <= intr_in( 53 );
         fold_intr_in( 16 ) <= intr_in( 54 );
         fold_intr_in( 17 ) <= intr_in( 55 );
         fold_intr_in( 18 ) <= intr_in( 56 );
        ELSE
         fold_intr_in( 0 ) <= "ZZZZZZZZZ";
         fold_intr_in( 1 ) <= "ZZZZZZZZZ";
         fold_intr_in( 2 ) <= "ZZZZZZZZZ";
         fold_intr_in( 3 ) <= "ZZZZZZZZZ";
         fold_intr_in( 4 ) <= "ZZZZZZZZZ";
         fold_intr_in( 5 ) <= "ZZZZZZZZZ";
         fold_intr_in( 6 ) <= "ZZZZZZZZZ";
         fold_intr_in( 7 ) <= "ZZZZZZZZZ";
         fold_intr_in( 8 ) <= "ZZZZZZZZZ";
         fold_intr_in( 9 ) <= "ZZZZZZZZZ";
         fold_intr_in( 10 ) <= "ZZZZZZZZZ";
         fold_intr_in( 11 ) <= "ZZZZZZZZZ";
         fold_intr_in( 12 ) <= "ZZZZZZZZZ";
         fold_intr_in( 13 ) <= "ZZZZZZZZZ";
         fold_intr_in( 14 ) <= "ZZZZZZZZZ";
         fold_intr_in( 15 ) <= "ZZZZZZZZZ";
         fold_intr_in( 16 ) <= "ZZZZZZZZZ";
         fold_intr_in( 17 ) <= "ZZZZZZZZZ";
         fold_intr_in( 18 ) <= "ZZZZZZZZZ";
        END IF;
      END IF;
    END IF;
END PROCESS;
----------------process for codeword convergence test ---------------------
PROCESS( decide_cword, en_sequencing )
BEGIN
    IF( en_sequencing = '1' ) THEN
      IF decide_cword = '1' THEN
        IF( pcheck_out
 = "000000000000000000000000000000000000000000000000000000000" ) THEN
          valid_word <= '1';
        ELSE
          valid_word <= '0';
        END IF;
      ELSE
        valid_word <= '0';  -- to avoid latch generation
      END IF;
    ELSE
      valid_word  <= '0';
    END IF;
END PROCESS;

---------------process for iteration counting------------------
PROCESS( en_sequencing, ctrl_store_addr )
BEGIN
    IF( en_sequencing = '0' ) THEN
      num_of_iter <= 0;
    ELSIF( en_sequencing = '1' AND ctrl_store_addr
 = "00000" ) THEN
      num_of_iter <= num_of_iter + 1;      -- Increase the iteration no.
    -- ELSE, num_of_iter will remain latched in this duration
    END IF;

END PROCESS;

--------------process for controlling sequencer ---------------
PROCESS(en_decode, valid_word)
BEGIN
      IF( en_decode = '1' AND valid_word = '1' ) THEN
        en_sequencing <= '0';
      ELSIF( en_decode = '1' ) THEN
        en_sequencing <= '1';
      ELSE
        en_sequencing <= '0'; -- initial few cycles may be
      END IF;
END PROCESS;

no_iter     <= num_of_iter;
output      <= rxvd_code;
valid_cword <= valid_word;

--------------- process to start/stop microcode sequencing----------------
PROCESS(clk)
BEGIN
    IF (clk'event AND clk='1') THEN
      IF ( en_sequencing = '1' ) THEN -- already factors valid_word != 0
        en_mem          <= '1';
      ELSE
        en_mem          <= '0';
      END IF;
    END IF;
END PROCESS;

END ARCHITECTURE Behavioral;
