LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE WORK.fold_package.ALL;

ENTITY h_p_switch IS
    PORT ( clk          : IN STD_LOGIC;
           en_p_switch  : IN STD_LOGIC;
           pat_id       : IN pat_indx;
           fu_id        : IN node_indx;

           in_val_1      : IN STD_LOGIC;
           in_val_2      : IN STD_LOGIC;
           in_val_3      : IN STD_LOGIC;
           in_val_4      : IN STD_LOGIC;
           in_val_5      : IN STD_LOGIC;
           in_val_6      : IN STD_LOGIC;
           in_val_7      : IN STD_LOGIC;
           in_val_8      : IN STD_LOGIC;
           out_val_1        : OUT STD_LOGIC;
           out_val_2        : OUT STD_LOGIC
         );
END ENTITY h_p_switch;

ARCHITECTURE behav OF h_p_switch IS

COMPONENT h_mux IS
    PORT ( clk          : IN STD_LOGIC;
           line_sel1    : IN INTEGER;
           line_sel2    : IN INTEGER;
           mux_id       : IN node_indx;
           en_mux       : IN STD_LOGIC;

           in_val_1      : IN STD_LOGIC;
           in_val_2      : IN STD_LOGIC;
           in_val_3      : IN STD_LOGIC;
           in_val_4      : IN STD_LOGIC;
           in_val_5      : IN STD_LOGIC;
           in_val_6      : IN STD_LOGIC;
           in_val_7      : IN STD_LOGIC;
           in_val_8      : IN STD_LOGIC;
           out_val_1        : OUT STD_LOGIC;
           out_val_2        : OUT STD_LOGIC
         );
END COMPONENT;

COMPONENT p_2_h_rom
    PORT( pat_id : IN pat_indx;
          out1   : OUT rom_sel_bus;
          out2   : OUT rom_sel_bus
        );
END COMPONENT;

FOR ALL:h_mux           USE ENTITY WORK.h_mux( behav );
FOR ALL:p_2_h_rom       USE ENTITY WORK.p_2_h_rom( behav );

    SIGNAL sel1, sel2: rom_sel_bus;

SIGNAL local_sel1, local_sel2: INTEGER range -1 to 21;
BEGIN

-- Selecting lines asynchronously helps in doing muxing in just 1 cycle.
a1: p_2_h_rom PORT MAP ( pat_id, sel1, sel2 );
a2: h_mux     PORT MAP ( clk, local_sel1, local_sel2, fu_id, en_p_switch,

 in_val_1, in_val_2, in_val_3, in_val_4, in_val_5, in_val_6, in_val_7, in_val_8, out_val_1, out_val_2 );

PROCESS( sel1 )
BEGIN
    IF( To_StdULogicVector(sel1) /= "ZZZZZ" ) THEN
        local_sel1 <= ( CONV_INTEGER(sel1) + CONV_INTEGER(fu_id) ) MOD 19;
    ELSE
        local_sel1 <= -1;
    END IF;
END PROCESS;

PROCESS( sel2 )
BEGIN
    IF( To_StdULogicVector(sel2) /= "ZZZZZ" ) THEN
        local_sel2 <= ( CONV_INTEGER(sel2) + CONV_INTEGER(fu_id) ) MOD 19;
    ELSE
        local_sel2 <= -1;
    END IF;
END PROCESS;

END ARCHITECTURE behav;
