LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE WORK.fold_package.ALL;

ENTITY p_m_switch IS
    PORT ( clk          : IN STD_LOGIC;
           en_m_switch  : IN STD_LOGIC;
           pat_id       : IN pat_indx;
           fu_id        : IN node_indx;
           in_val1      : IN STD_LOGIC;
           in_val2      : IN STD_LOGIC;

           out_val_1     : OUT STD_LOGIC;
           out_val_2     : OUT STD_LOGIC;
           out_val_3     : OUT STD_LOGIC;
           out_val_4     : OUT STD_LOGIC;
           out_val_5     : OUT STD_LOGIC;
           out_val_6     : OUT STD_LOGIC;
           out_val_7     : OUT STD_LOGIC;
           out_val_8     : OUT STD_LOGIC
        );
END ENTITY p_m_switch;

ARCHITECTURE behav OF p_m_switch IS

COMPONENT p_demux
    PORT ( clk          : IN STD_LOGIC;
           line_sel1    : IN INTEGER;
           line_sel2    : IN INTEGER;
           en_demux     : IN STD_LOGIC;
           demux_id     : IN node_indx;
           in_val1      : IN STD_LOGIC;
           in_val2      : IN STD_LOGIC;

           out_val_1     : OUT STD_LOGIC;
           out_val_2     : OUT STD_LOGIC;
           out_val_3     : OUT STD_LOGIC;
           out_val_4     : OUT STD_LOGIC;
           out_val_5     : OUT STD_LOGIC;
           out_val_6     : OUT STD_LOGIC;
           out_val_7     : OUT STD_LOGIC;
           out_val_8     : OUT STD_LOGIC
        );
END COMPONENT;

COMPONENT p_2_h_inv_rom
    PORT( pat_id : IN pat_indx;
          out1   : OUT rom_sel_bus;
          out2   : OUT rom_sel_bus
        );
END COMPONENT;

FOR ALL:p_demux         USE ENTITY WORK.p_demux( behav );
FOR ALL:p_2_h_inv_rom   USE ENTITY WORK.p_2_h_inv_rom( behav );

    SIGNAL sel1, sel2: rom_sel_bus;

SIGNAL local_sel1, local_sel2: INTEGER range -1 to 21;
BEGIN

-- Selecting lines asynchronously helps in doing muxing in just 1 cycle.
a1: p_2_h_inv_rom   PORT MAP ( pat_id, sel1, sel2 );
a2: p_demux       PORT MAP ( clk, local_sel1, local_sel2, en_m_switch, fu_id,
                             in_val1, in_val2,
 out_val_1, out_val_2, out_val_3, out_val_4, out_val_5, out_val_6, out_val_7, out_val_8);

PROCESS( sel1 )
BEGIN
    IF( To_StdULogicVector(sel1) /= "ZZZZZ" ) THEN
        local_sel1 <= ( CONV_INTEGER(sel1) + CONV_INTEGER(fu_id) ) MOD 19;
    ELSE
        local_sel1 <= -1;
    END IF;
END PROCESS;

PROCESS( sel2 )
BEGIN
    IF( To_StdULogicVector(sel2) /= "ZZZZZ" ) THEN
        local_sel2 <= ( CONV_INTEGER(sel2) + CONV_INTEGER(fu_id) ) MOD 19;
    ELSE
        local_sel2 <= -1;
    END IF;
END PROCESS;

END ARCHITECTURE behav;
