LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;  -- CONV_INTEGER()
USE WORK.fold_package.ALL;

ENTITY tdp_mem IS
PORT (  clk     : IN STD_LOGIC;
        rst     : IN STD_LOGIC;
        en_mem  : IN STD_LOGIC;
        rwbar_1 : IN STD_LOGIC;
        rwbar_2 : IN STD_LOGIC;
        addr1   : IN mem_addr_bus;
        addr2   : IN mem_addr_bus;
        din_1   : IN STD_LOGIC;
        din_2   : IN STD_LOGIC;
        dout_1  : OUT STD_LOGIC;
        dout_2  : OUT STD_LOGIC
     );
END ENTITY tdp_mem;

ARCHITECTURE behav OF tdp_mem IS
        -- The size of memory is
        -- { 1 bit wide x (Order + [Order % 2]) x Fold_Factor }
        -- locations deep.

TYPE RAM_TYPE IS ARRAY (24 DOWNTO 0) OF STD_LOGIC;
        SIGNAL ram : RAM_TYPE;
BEGIN
PROCESS( clk, rst )
BEGIN
    IF( rst = '1' ) THEN
        L1: FOR i in 0 to 24 LOOP
            ram(i) <= '0';
        END LOOP L1;
    ELSIF (clk'EVENT AND clk = '1') THEN
        IF( en_mem = '1' ) THEN
            IF ( rwbar_1 = '1') THEN
              dout_1  <= ram( conv_integer(addr1) );
            ELSIF ( rwbar_1 = '0') THEN
              ram( conv_integer(addr1) ) <= din_1;
            END IF;

            IF ( rwbar_2 = '1' ) THEN
              dout_2  <= ram( conv_integer(addr2) );
            ELSIF ( rwbar_2 = '0' ) THEN
              ram( conv_integer(addr2) ) <= din_2;
            END IF;
        END IF;
    END IF;
END PROCESS;

END ARCHITECTURE behav;
